Package: bayesImageS
Type: Package
Title: Bayesian Methods for Image Segmentation using a Potts Model
Version: 0.7-0
Date: 2025-10-10
Encoding: UTF-8
Authors@R: c(
    person("Matt", "Moores", role = c("aut", "cre", "cph"), email = "mmoores@gmail.com", comment = c(ORCID = "0000-0003-4531-3572")),
    person("Dai", "Feng", role="ctb"),
    person("Kerrie", "Mengersen", role=c("aut", "ths"), email="k.mengersen@qut.edu.au", comment = c(ORCID = "0000-0001-8625-9168")))
Description: Various algorithms for segmentation of 2D and 3D images, such
    as computed tomography and satellite remote sensing. This package implements
    Bayesian image analysis using the hidden Potts model with external field
    prior of Moores et al. (2015) <doi:10.1016/j.csda.2014.12.001>.
    Latent labels are sampled using chequerboard updating or Swendsen-Wang.
    Algorithms for the smoothing parameter include pseudolikelihood, path sampling,
    the exchange algorithm, approximate Bayesian computation (ABC-MCMC and ABC-SMC),
    and the parametric functional approximate Bayesian (PFAB) algorithm. Refer to
    Moores, Pettitt & Mengersen (2020) <doi:10.1007/978-3-030-42553-1_6> for an overview
    and also to <doi:10.1007/s11222-014-9525-6> and <doi:10.1214/18-BA1130> for
    further details of specific algorithms.
License: GPL (>= 2) | file LICENSE
URL: https://bitbucket.org/Azeari/bayesimages,
        https://mooresm.github.io/bayesImageS/
BugReports: https://bitbucket.org/Azeari/bayesimages/jira
LazyData: true
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.12)
LinkingTo: Rcpp, RcppArmadillo
Suggests: mcmcse, coda, PottsUtils, rstan, knitr, rmarkdown, lattice
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-10-10 02:06:16 UTC; mmoores
Author: Matt Moores [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4531-3572>),
  Dai Feng [ctb],
  Kerrie Mengersen [aut, ths] (ORCID:
    <https://orcid.org/0000-0001-8625-9168>)
Maintainer: Matt Moores <mmoores@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-10 08:30:11 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-25 10:01:26 UTC; unix
Archs: bayesImageS.so.dSYM

# Generated by roxygen2: do not edit by hand

export(exactPotts)
export(getBlocks)
export(getEdges)
export(getNeighbors)
export(gibbsGMM)
export(gibbsNorm)
export(gibbsPotts)
export(initSedki)
export(mcmcPotts)
export(mcmcPottsNoData)
export(smcPotts)
export(sufficientStat)
export(swNoData)
export(testResample)
importFrom(Rcpp,sourceCpp)
useDynLib(bayesImageS)

bayesImageS
================

<!--- README.md is generated from README.Rmd. Please edit that file -->

[![cran
version](https://www.r-pkg.org/badges/version/bayesImageS)](https://cran.r-project.org/package=bayesImageS)
[![rstudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/bayesImageS)](https://github.com/r-hub/cranlogs.app)

<img src="inst/image/README-logo.png" alt="" width="100px" height="100px" style="display: block; margin: auto 0 auto auto;" />

`bayesImageS` implements algorithms for segmentation of 2D and 3D
images, such as computed tomography (CT) and satellite remote sensing.
This R package provides functions for Bayesian image analysis using a
hidden Potts/Ising model with external field prior. Latent labels are
updated using chequerboard Gibbs sampling or Swendsen-Wang. Algorithms
for the smoothing parameter include:

- pseudolikelihood
- path sampling (thermodynamic integration)
- approximate exchange algorithm (AEA)
- approximate Bayesian computation (ABC-MCMC and ABC-SMC)
- Bayesian indirect likelihood (BIL), including the parametric
  functional approximate Bayesian (PFAB) algorithm

# Installation Instructions

Stable releases, including binary packages for Windows & Mac OS, are
available from CRAN:

- <https://CRAN.R-project.org/package=bayesImageS>

``` r
install.packages("bayesImageS")
```

The current development version can be installed from Bitbucket:

``` r
devtools::install_git("https://bitbucket.org/Azeari/bayesimages/")
```

# Example Usage

To generate synthetic data for a known value of $\beta$:

``` r
set.seed(1234)
mask <- matrix(1,3,3)
neigh <- getNeighbors(mask, c(2,2,0,0))
blocks <- getBlocks(mask, 2)
k <- 3
beta <- 0.7
res.sw <- swNoData(beta, k, neigh, blocks, niter=200)
z <- matrix(max.col(res.sw$z)[1:nrow(neigh)], nrow=nrow(mask))
image(z, xaxt = 'n', yaxt='n', col=rainbow(k), asp=1)
```

![](inst/image/README-unnamed-chunk-5-1.png)<!-- -->

Now add some Gaussian noise to the labels, according to the prior:

``` r
priors <- list()
priors$k <- k
priors$mu <- c(-2, 0, 2)
priors$mu.sd <- rep(0.5,k)
priors$sigma <- rep(0.25,k)
priors$sigma.nu <- rep(3, k)
priors$beta <- c(0,1.3*log(1 + sqrt(k)))

m0 <- sort(rnorm(priors$k,priors$mu,priors$mu.sd))
SS0 <- priors$sigma.nu*priors$sigma^2
s0 <- 1/sqrt(rgamma(priors$k,priors$sigma.nu/2,SS0/2))
l <- as.vector(z)
y <- m0[l] + rnorm(nrow(neigh),0,s0[l])
library(lattice)
levelplot(matrix(y, nrow=nrow(mask)))
```

![](inst/image/README-unnamed-chunk-6-1.png)<!-- -->

Image segmentation using ABC-SMC:

``` r
res.smc <- smcPotts(y, neigh, blocks, priors=priors)
#> Initialization took 39sec
#> Iteration 1
#> previous epsilon 7 and ESS 10000 (target: 9500)
#> Took 0sec to update epsilon=2.625 (ESS=9505.29)
#> Took 49sec for 8918 RWMH updates (bw=0.497509)
#> Took 4sec for 10000 iterations to calculate S(z)=7
#> Iteration 2
#> previous epsilon 2.625 and ESS 9505.29 (target: 9030.02)
#> Took 5sec to update epsilon=1 (ESS=7970.86)
#> Took 36sec for 7671 RWMH updates (bw=0.466951)
#> Took 4sec for 10000 iterations to calculate S(z)=6
#> Iteration 3
#> previous epsilon 1 and ESS 7970.86 (target: 7572.32)
#> Took 5sec to update epsilon=4.66632e-302 (ESS=7949.67)
#> Took 71sec for 7968 RWMH updates (bw=0.466673)
#> Took 14sec for 10000 iterations to calculate S(z)=7
# pixel classifications
pred <- res.smc$alloc/rowSums(res.smc$alloc)
predMx <- as.raster(array(pred, dim=c(nrow(mask),ncol(mask),3)))
plot(c(0.5,3.5),c(0.5,3.5),type='n',xaxt='n',yaxt='n',xlab="",ylab="",asp=1)
rasterImage(t(predMx)[nrow(mask):1,], 0.5, 0.5, 3.5, 3.5, interpolate = FALSE)
```

![](inst/image/README-unnamed-chunk-7-1.png)<!-- -->

Note that CODA ignores the particle weights, so we need to resample to
obtain accurate HPD intervals. This step is not usually necessary and
does introduce some noise due to duplication of particles. Depending on
how many SMC iterations have been performed, one or more resampling
steps might have already been done (but not in this specific example).

``` r
seg <- max.col(res.smc$alloc) # posterior mode (0-1 loss)
all.equal(seg, l)
#> [1] TRUE

# filter weights to remove Ninf, NaN
w <- res.smc$wt
w[is.na(w)] <- 0
hist(res.smc$beta, main=expression(paste("Posterior for ",beta)),
     freq=FALSE, col=3, xlab=expression(beta), breaks=15)
lines(density(res.smc$beta, weights=w), lwd=3, lty=2, col=4)
abline(v=log(1 + sqrt(k)),lty=3,col=2,lwd=2) # critical point
rug(beta, col=4, lwd=3)
```

![](inst/image/README-unnamed-chunk-8-1.png)<!-- -->

``` r

library(coda)
res.res <- testResample(res.smc$beta, w, cbind(res.smc$mu, res.smc$sigma))
#> Took 5sec to resample 10000 particles
res.coda <- mcmc(cbind(res.res$pseudo, res.res$beta))
varnames(res.coda) <- c(paste("mu",1:k), paste("sd",1:k), "beta")
HPDinterval(res.coda)
#>            lower      upper
#> mu 1 -2.56248597 -1.9038983
#> mu 2 -0.50953511  0.4881316
#> mu 3  1.37933533  2.7939284
#> sd 1  0.11321785  0.4984628
#> sd 2  0.22470066  0.9154286
#> sd 3  0.11011734  0.9105506
#> beta  0.09338037  1.2852256
#> attr(,"Probability")
#> [1] 0.95
m0
#> [1] -2.279614  0.156580  2.208122
s0
#> [1] 0.2785175 0.6092555 0.3153176
beta
#> [1] 0.7
```

bayesImageS
================

<!--- README.md is generated from README.Rmd. Please edit that file -->

[![cran
version](https://www.r-pkg.org/badges/version/bayesImageS)](https://cran.r-project.org/package=bayesImageS)
[![rstudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/bayesImageS)](https://github.com/r-hub/cranlogs.app)

<img src="inst/image/README-logo.png" width="100px" height="100px" style="display: block; margin: auto 0 auto auto;" />

`bayesImageS` implements algorithms for segmentation of 2D and 3D
images, such as computed tomography (CT) and satellite remote sensing.
This R package provides functions for Bayesian image analysis using a
hidden Potts/Ising model with external field prior. Latent labels are
updated using chequerboard Gibbs sampling or Swendsen-Wang. Algorithms
for the smoothing parameter include:

- pseudolikelihood
- path sampling (thermodynamic integration)
- approximate exchange algorithm (AEA)
- approximate Bayesian computation (ABC-MCMC and ABC-SMC)
- Bayesian indirect likelihood (BIL), including the parametric
  functional approximate Bayesian (PFAB) algorithm

# Installation Instructions

Stable releases, including binary packages for Windows & Mac OS, are
available from CRAN:

- <https://CRAN.R-project.org/package=bayesImageS>

``` r
install.packages("bayesImageS")
```

The current development version can be installed from Bitbucket:

``` r
devtools::install_git("https://bitbucket.org/Azeari/bayesimages/")
```

# Example Usage

To generate synthetic data for a known value of $\beta$:

``` r
set.seed(1234)
mask <- matrix(1,3,3)
neigh <- getNeighbors(mask, c(2,2,0,0))
blocks <- getBlocks(mask, 2)
k <- 3
beta <- 0.7
res.sw <- swNoData(beta, k, neigh, blocks, niter=200)
z <- matrix(max.col(res.sw$z)[1:nrow(neigh)], nrow=nrow(mask))
image(z, xaxt = 'n', yaxt='n', col=rainbow(k), asp=1)
```

![](inst/image/README-unnamed-chunk-5-1.png)<!-- -->

Now add some Gaussian noise to the labels, according to the prior:

``` r
priors <- list()
priors$k <- k
priors$mu <- c(-2, 0, 2)
priors$mu.sd <- rep(0.5,k)
priors$sigma <- rep(0.25,k)
priors$sigma.nu <- rep(3, k)
priors$beta <- c(0,1.3*log(1 + sqrt(k)))

m0 <- sort(rnorm(priors$k,priors$mu,priors$mu.sd))
SS0 <- priors$sigma.nu*priors$sigma^2
s0 <- 1/sqrt(rgamma(priors$k,priors$sigma.nu/2,SS0/2))
l <- as.vector(z)
y <- m0[l] + rnorm(nrow(neigh),0,s0[l])
library(lattice)
levelplot(matrix(y, nrow=nrow(mask)))
```

![](inst/image/README-unnamed-chunk-6-1.png)<!-- -->

Image segmentation using ABC-SMC:

``` r
res.smc <- smcPotts(y, neigh, blocks, priors=priors)
#> Initialization took 66sec
#> Iteration 1
#> previous epsilon 7 and ESS 10000 (target: 9500)
#> Took 0sec to update epsilon=2.625 (ESS=9505.29)
#> Took 58sec for 8918 RWMH updates (bw=0.497509)
#> Took 4sec for 10000 iterations to calculate S(z)=7
#> Iteration 2
#> previous epsilon 2.625 and ESS 9505.29 (target: 9030.02)
#> Took 7sec to update epsilon=1 (ESS=7970.86)
#> Took 55sec for 7671 RWMH updates (bw=0.466951)
#> Took 4sec for 10000 iterations to calculate S(z)=6
#> Iteration 3
#> previous epsilon 1 and ESS 7970.86 (target: 7572.32)
#> Took 7sec to update epsilon=4.66632e-302 (ESS=7949.67)
#> Took 55sec for 7968 RWMH updates (bw=0.466673)
#> Took 4sec for 10000 iterations to calculate S(z)=7
# pixel classifications
pred <- res.smc$alloc/rowSums(res.smc$alloc)
predMx <- as.raster(array(pred, dim=c(nrow(mask),ncol(mask),3)))
plot(c(0.5,3.5),c(0.5,3.5),type='n',xaxt='n',yaxt='n',xlab="",ylab="",asp=1)
rasterImage(t(predMx)[nrow(mask):1,], 0.5, 0.5, 3.5, 3.5, interpolate = FALSE)
```

![](inst/image/README-unnamed-chunk-7-1.png)<!-- -->

Note that CODA ignores the particle weights, so we need to resample to
obtain accurate HPD intervals. This step is not usually necessary and
does introduce some noise due to duplication of particles. Depending on
how many SMC iterations have been performed, one or more resampling
steps might have already been done (but not in this specific example).

``` r
seg <- max.col(res.smc$alloc) # posterior mode (0-1 loss)
all.equal(seg, l)
#> [1] TRUE

# filter weights to remove Ninf, NaN
w <- res.smc$wt
w[is.na(w)] <- 0
plot(density(res.smc$beta, weights=w),main=expression(paste("Posterior for ",beta)))
abline(h=0,lty=3)
abline(v=beta,lty=2,col=4)
abline(v=log(1 + sqrt(k)),lty=3,col=2) # critical point
```

![](inst/image/README-unnamed-chunk-8-1.png)<!-- -->

``` r

library(coda)
res.res <- testResample(res.smc$beta, w, cbind(res.smc$mu, res.smc$sigma))
#> Took 1sec to resample 10000 particles
res.coda <- mcmc(cbind(res.res$pseudo, res.res$beta))
varnames(res.coda) <- c(paste("mu",1:k), paste("sd",1:k), "beta")
HPDinterval(res.coda)
#>            lower      upper
#> mu 1 -2.56248597 -1.9038983
#> mu 2 -0.50953511  0.4881316
#> mu 3  1.37933533  2.7939284
#> sd 1  0.11321785  0.4984628
#> sd 2  0.22470066  0.9154286
#> sd 3  0.11011734  0.9105506
#> beta  0.09338037  1.2852256
#> attr(,"Probability")
#> [1] 0.95
m0
#> [1] -2.279614  0.156580  2.208122
s0
#> [1] 0.2785175 0.6092555 0.3153176
beta
#> [1] 0.7
```

## -----------------------------------------------------------------------------
beta <- seq(0,2,by=0.1)
tmMx.PU <- tmMx.bIS <- matrix(nrow=length(beta),ncol=2)
rownames(tmMx.PU) <- rownames(tmMx.bIS) <- beta
colnames(tmMx.PU) <- colnames(tmMx.bIS) <- c("user","elapsed")

## -----------------------------------------------------------------------------
iter <- 600
burn <- 100
samp.PU <- samp.bIS <- matrix(nrow=length(beta),ncol=iter-burn)

## -----------------------------------------------------------------------------
library(bayesImageS)

mask <- matrix(1,50,50)
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))

n <- sum(mask)
k <- 2
bcrit <- log(1 + sqrt(k))
maxSS <- nrow(edges)

for (i in 1:length(beta)) {
  if (requireNamespace("PottsUtils", quietly = TRUE)) {
    tm <- system.time(result <- PottsUtils::SW(iter,n,k,edges,beta=beta[i]))
    tmMx.PU[i,"user"] <- tm["user.self"]
    tmMx.PU[i,"elapsed"] <- tm["elapsed"]
    res <- sufficientStat(result, neigh, block, k)
    samp.PU[i,] <- res$sum[(burn+1):iter]
    print(paste("PottsUtils::SW",beta[i],tm["elapsed"],median(samp.PU[i,])))
   } else {
      print("PottsUtils::SW unavailable on this platform.")
   }

  
  # bayesImageS
  tm <- system.time(result <- swNoData(beta[i],k,neigh,block,iter))
  tmMx.bIS[i,"user"] <- tm["user.self"]
  tmMx.bIS[i,"elapsed"] <- tm["elapsed"]
  samp.bIS[i,] <- result$sum[(burn+1):iter]
  print(paste("bayesImageS::swNoData",beta[i],tm["elapsed"],median(samp.bIS[i,])))
}

## -----------------------------------------------------------------------------
summary(tmMx.PU)
summary(tmMx.bIS)
boxplot(tmMx.PU[,"elapsed"],tmMx.bIS[,"elapsed"],ylab="seconds elapsed",names=c("SW","swNoData"))

## -----------------------------------------------------------------------------
s_z <- c(samp.PU,samp.bIS)
s_x <- rep(beta,times=iter-burn)
s_a <- rep(1:2,each=length(beta)*(iter-burn))
s.frame <- data.frame(s_z,c(s_x,s_x),s_a)
names(s.frame) <- c("stat","beta","alg")
s.frame$alg <- factor(s_a,labels=c("SW","swNoData"))
  if (requireNamespace("lattice", quietly = TRUE)) {
    lattice::xyplot(stat ~ beta | alg, data=s.frame)
  }
plot(c(s_x,s_x),s_z,pch=s_a,xlab=expression(beta),ylab=expression(S(z)))
abline(v=bcrit,col="red")

## -----------------------------------------------------------------------------
rowMeans(samp.bIS) - rowMeans(samp.PU)
apply(samp.PU, 1, sd)
apply(samp.bIS, 1, sd)
s.frame$beta <- factor(c(s_x,s_x))
  if (requireNamespace("PottsUtils", quietly = TRUE)) {
    s.fit <- aov(stat ~ alg + beta, data=s.frame)
    summary(s.fit)
    TukeyHSD(s.fit,which="alg")
  }

## ----setup, include=FALSE-----------------------------------------------------
library(bayesImageS)
set.seed(12)

## ----fig.cap="Chequerboard Gibbs sampling for a regular 8x8 lattice with k=3 labels."----
k <- 3
n <- 8*8
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.Gibbs <- mcmcPottsNoData(beta=5, k=3, neigh, block, niter=100)
ts.plot(res.Gibbs$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.Gibbs$sum[51:100])
var(res.Gibbs$sum[51:100])

## ----fig.cap="Chequerboard Gibbs sampling for a regular 125x125 lattice with k=3 labels."----
n <- 125*125
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.Gibbs <- mcmcPottsNoData(beta=5, k=3, neigh, block, niter=2000)
ts.plot(res.Gibbs$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.Gibbs$sum[1001:2000])
var(res.Gibbs$sum[1001:2000])

## -----------------------------------------------------------------------------
res.SW <- swNoData(beta=5, k=3, neigh, block, niter=50)
ts.plot(res.SW$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.SW$sum[26:50])
var(res.SW$sum[26:50])

## -----------------------------------------------------------------------------
n <- 1000*1000
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.SW <- swNoData(beta=5, k=3, neigh, block, niter=100)
ts.plot(res.SW$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.SW$sum[81:100])
var(res.SW$sum[81:100])

## -----------------------------------------------------------------------------
n <- 125*125
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
system.time(res2.Gibbs <- mcmcPottsNoData(beta=0, k=3, neigh, block, niter=100, random=FALSE))
ts.plot(res2.Gibbs$sum, ylim=range(c(res2.Gibbs$sum, nrow(edges))))
abline(h=nrow(edges), col=2, lty=3)
abline(h=nrow(edges)/3, col=4, lty=3)
summary(res2.Gibbs$sum[51:100])
var(res2.Gibbs$sum[51:100])

## -----------------------------------------------------------------------------
muSz <- nrow(edges)/3
sdSz <- sqrt(nrow(edges)*(1/3)*(2/3))
hist(res2.Gibbs$sum[51:100], freq=FALSE, breaks=20, col=3,
     xlim=range(c(res2.Gibbs$sum,muSz - 3*sdSz, muSz + 3*sdSz)))
abline(v=nrow(edges)/3, col=4, lty=3, lwd=3)
curve(dnorm(x, mean=nrow(edges)/3, sd=sqrt(nrow(edges)*(1/3)*(2/3))), 
          col="darkblue", lwd=2, lty=2, add=TRUE, yaxt="n")

## -----------------------------------------------------------------------------
system.time(res2.SW <- swNoData(beta=0, k=3, neigh, block, niter=100, random=FALSE))
ts.plot(res2.SW$sum, ylim=range(c(res2.SW$sum, nrow(edges))))
abline(h=nrow(edges), col=2, lty=3)
abline(h=nrow(edges)/3, col=4, lty=3)
summary(res2.SW$sum[51:100])
var(res2.SW$sum[51:100])

## -----------------------------------------------------------------------------
hist(res2.SW$sum[51:100], freq=FALSE, breaks=20, col=3,
     xlim=range(c(res2.Gibbs$sum,muSz - 3*sdSz, muSz + 3*sdSz)))
abline(v=nrow(edges)/3, col=4, lty=3, lwd=3)
curve(dnorm(x, mean=muSz, sd=sdSz), 
          col="darkblue", lwd=2, lty=2, add=TRUE, yaxt="n")

#' Fit the hidden Potts model using approximate Bayesian computation with sequential Monte Carlo (ABC-SMC).
#' 
#' @param y A vector of observed pixel data.
#' @param neighbors A matrix of all neighbors in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param param A list of options for the ABC-SMC algorithm.
#' @param priors A list of priors for the parameters of the model.
#' @return A matrix containing SMC samples for the parameters of the Potts model.
#' @export
smcPotts <- function(y, neighbors, blocks, param=list(npart=10000, nstat=50), priors=NULL) {
  result <- .Call( "smcPotts", y, neighbors, blocks, param, priors, PACKAGE = "bayesImageS")
}

#' Initialize the ABC algorithm using the method of Sedki et al. (2013)
#' 
#' @param y A vector of observed pixel data.
#' @param neighbors A matrix of all neighbours in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param param A list of options for the ABC-SMC algorithm.
#' @param priors A list of priors for the parameters of the model.
#' @return A matrix containing SMC samples for the parameters of the Potts model.
#' @export
#' @references
#' Sedki, M.; Pudlo, P.; Marin, J.-M.; Robert, C. P. & Cornuet, J.-M. (2013) "Efficient learning in ABC algorithms" \href{https://arxiv.org/abs/1210.1388}{arXiv:1210.1388}
initSedki <- function(y, neighbors, blocks, param=list(npart=10000), priors=NULL) {
  result <- .Call( "initSedki", y, neighbors, blocks, param, priors, PACKAGE = "bayesImageS")
}

#' Test the residual resampling algorithm.
#' 
#' @param values A vector of SMC particles.
#' @param weights A vector of importance weights for each particle.
#' @param pseudo A matrix of pseudo-data for each particle.
#' @return A list containing the following elements:
#' \describe{
#'   \item{\code{beta}}{A vector of resampled particles.}
#'   \item{\code{wt}}{The new importance weights, after resampling.}
#'   \item{\code{pseudo}}{A matrix of pseudo-data for each particle.}
#'   \item{\code{idx}}{The indices of the parents of the resampled particles.}
#'   }
#' @export
#' @references 
#'  Liu, J. S. & Chen, R. (1998) "Sequential Monte Carlo Methods for Dynamic Systems"
#'  \emph{J. Am. Stat. Assoc.} \bold{93}(443): 1032--1044, DOI: \doi{10.1080/01621459.1998.10473765}
testResample <- function(values, weights, pseudo) {
  result <- .Call( "testResample", values, weights, pseudo, PACKAGE = "bayesImageS")
}

#' Calculate the distribution of the Potts model using a brute force algorithm.
#' 
#' \bold{Warning}: this algorithm is O\eqn{(k^n)} and therefore will not scale for
#' \eqn{k^n > 2^{31} - 1}
#' 
#' @param neighbors A matrix of all neighbours in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param k The number of unique labels.
#' @param beta The inverse temperature parameter of the Potts model.
#' @return A list containing the following elements:
#' \describe{
#'   \item{\code{expectation}}{The exact mean of the sufficient statistic.}
#'   \item{\code{variance}}{The exact variance of the sufficient statistic.}
#'   \item{\code{exp_PL}}{Pseudo-likelihood (PL) approximation of the expectation of S(z).}
#'   \item{\code{var_PL}}{PL approx. of the variance of the sufficient statistic.}
#'   }
#' @export
exactPotts <- function(neighbors, blocks, k, beta) {
  result <- .Call( "exactPotts", neighbors, blocks, k, beta, PACKAGE = "bayesImageS")
}

#' Package bayesImageS
#'
#' Bayesian methods for segmentation of 2D and 3D images, such as computed
#' tomography and satellite remote sensing. This package implements image
#' analysis using the hidden Potts model with external field prior. Latent labels
#' are sampled using chequerboard updating or Swendsen-Wang. Algorithms for the 
#' smoothing parameter include pseudolikelihood, path sampling, the exchange
#' algorithm, and approximate Bayesian computation (ABC-MCMC and ABC-SMC).
#' 
#' @author
#' M. T. Moores and K. Mengersen
#' with additional code contributed by D. Feng
#' 
#' Maintainer: Matt Moores <mmoores@gmail.com>
#' 
#' @references
#' Moores, M. T.; Nicholls, G. K.; Pettitt, A. N. & Mengersen, K. (2020) "Scalable Bayesian inference for the inverse temperature of a hidden Potts model" \emph{Bayesian Analysis} \bold{15}(1), 1--17, DOI: \doi{10.1214/18-BA1130}
#' 
#' Moores, M. T.; Drovandi, C. C.; Mengersen, K. & Robert, C. P. (2015) "Pre-processing for approximate Bayesian computation in image analysis" \emph{Statistics & Computing} \bold{25}(1), 23--33, DOI: \doi{10.1007/s11222-014-9525-6}
#' 
#' Moores, M. T.; Hargrave, C. E.; Deegan, T.; Poulsen, M.; Harden, F. & Mengersen, K. (2015) "An external field prior for the hidden Potts model, with application to cone-beam computed tomography" \emph{Computational Statistics & Data Analysis} \bold{86}, 27--41, DOI: \doi{10.1016/j.csda.2014.12.001}
#' 
#' Feng, D. (2008) "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification" \emph{Ph. D. Dissertation, The University of Iowa} 
#'
#' @seealso
#' \code{vignette(package="bayesImageS")}
#'
#' @useDynLib bayesImageS
#' @importFrom Rcpp sourceCpp
#' @name bayesImageS
#' @keywords internal
"_PACKAGE"

.onUnload <- function (libpath) {
  library.dynam.unload("bayesImageS", libpath)
}

#' Simulation from the Potts model using Swendsen-Wang.
#' 
#' Simulations for a \eqn{500 \times 500} lattice for fixed values
#' of the inverse temperature parameter, \eqn{\beta}.
#' 
#' @format A \code{list} containing 5 variables:
#' \describe{
#'   \item{0.22}{simulations for \eqn{\beta = 0.22}}
#'   \item{0.44}{simulations for \eqn{\beta = 0.44}}
#'   \item{0.88}{simulations for \eqn{\beta = 0.88}}
#'   \item{1.32}{simulations for \eqn{\beta = 1.32}}
#'   \item{tm}{time taken by the simulations}
#' }
#' @seealso \code{\link{swNoData}}
"synth"

#' Simulation from the Potts model using single-site Gibbs updates.
#' 
#' 100 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
#' with \eqn{\beta=0.22} and \eqn{k=2}.
#' @format A \code{list} containing 7 variables.
#' @seealso \code{\link{mcmcPotts}}
"res"

#' Simulation from the Potts model using single-site Gibbs updates.
#' 
#' 100 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
#' with \eqn{\beta=0.44} and \eqn{k=2}.
#' @format A \code{list} containing 7 variables.
#' @seealso \code{\link{mcmcPotts}}
"res2"

#' Simulation from the Potts model using single-site Gibbs updates.
#' 
#' 100 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
#' with \eqn{\beta=0.88} and \eqn{k=2}.
#' @format A \code{list} containing 7 variables.
#' @seealso \code{\link{mcmcPotts}}
"res3"

#' Simulation from the Potts model using single-site Gibbs updates.
#' 
#' 100 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
#' with \eqn{\beta=1.32} and \eqn{k=2}.
#' @format A \code{list} containing 7 variables.
#' @seealso \code{\link{mcmcPotts}}
"res4"

#' Simulation from the Potts model using single-site Gibbs updates.
#' 
#' 5000 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
#' with \eqn{\beta=1.32} and \eqn{k=2}.
#' @format A \code{list} containing 4 variables.
#' @seealso \code{\link{mcmcPottsNoData}}
"res5"

getBlocksLine <- function(mask, nblock){
    nvertex <- sum(mask == 1)
    lapply(1:nblock, function(x) seq(x, nvertex, by=nblock))
}

getBlocksPlane <- function(mask, nblock){
    ind <- which(mask == 1, arr.ind=T)
    oe1 <- ind[,1] %% 2
    oe2 <- ind[,2] %% 2
    if(nblock==2){
        mark <- abs(oe1 - oe2)
        lapply(1:2, function(x) which(mark == (x-1)))
    }
    else
        lapply(1:4, function(x) which((oe1 + oe2*2) == (x-1)))
}

getBlocksCube <- function(mask, nblock){
    ind <- which(mask == 1, arr.ind=T)
    x.mark <- ind[,1] %% 2
    y.mark <- ind[,2] %% 2
    z.mark <- ind[,3] %% 2

    if(nblock==2){
        BW <- ifelse(rowSums(cbind(x.mark, y.mark, z.mark)) %% 2 == 1, 1, 0)
        lapply(1:2, function(x) which(BW == (x-1)))
    }

    else
        lapply(1:8, function(x) which((x.mark + y.mark*2 + z.mark*4) == (x-1)))
}

#' Get Blocks of a Graph
#' 
#'   Obtain blocks of vertices of a 1D, 2D, or 3D graph, in order to use
#'   the conditional independence to speed up the simulation (chequerboard idea). 
#'
#' @param mask a vector, matrix, or 3D array specifying vertices of a graph. Vertices of value 1 are within the graph and 0 are not.
#' @param nblock a scalar specifying the number of blocks. For a 2D graph \code{nblock} could be either 2 or 4, and for a 3D graph \code{nblock} could be either 2 or 8.
#'
#' @return A list with the number of components equal to \code{nblock}. Each component consists of vertices within the same block.
#' @export
#' @details The vertices within each block are mutually independent given the vertices in other blocks. Some blocks could be empty.
#' @references
#'     Wilkinson, D. J. (2005)
#'     "Parallel Bayesian Computation"
#'     \cite{Handbook of Parallel Computing and Statistics}, pp. 481-512
#'     \emph{Marcel Dekker/CRC Press}
#' @keywords spatial
#' @usage getBlocks(mask, nblock)
#' @examples 
#'   #Example 1: split a line into 2 blocks
#'   getBlocks(mask=c(1,1,1,1,0,0,1,1,0), nblock=2)
#'   
#'   #Example 2: split a 4*4 2D graph into 4 blocks in order
#'   #           to use the chequerboard idea for a neighbourhood structure
#'   #           corresponding to the second-order Markov random field.
#'   getBlocks(mask=matrix(1, nrow=4, ncol=4), nblock=4)
#'   
#'   #Example 3: split a 3*3*3 3D graph into 8 blocks
#'   #           in order to use the chequerboard idea for a neighbourhood
#'   #           structure based on the 18 neighbors definition, where the
#'   #           neighbors of a vertex comprise its available
#'   #           adjacencies sharing the same edges or faces.
#'   mask <- array(1, dim=rep(3,3))
#'   getBlocks(mask, nblock=8)
getBlocks <- function(mask, nblock){
    if(!(is.vector(mask) || is.matrix(mask) || length(dim(mask))==3))
        stop("The graph has to be in 1D, 2D, or 3D.")
    if(nblock <= 0 || floor(nblock) != nblock)
        stop("The number of blocks has to be and integer bigger than 0")
    if(length(mask==1) < nblock)
        stop("The number of blocks has to be less than
             the number of vertices inside the mask.")
    if(is.matrix(mask) && !nblock %in% c(2, 4))
        stop("For a plane, the number of blocks has to be equal to 2 or 4.")
    if(length(dim(mask))==3 && !nblock %in% c(2, 8))
        stop("For a cube, the number of blocks has to be equal to 2 or 8.")

    if(is.vector(mask))
        getBlocksLine(mask, nblock)

    else if(is.matrix(mask))
        getBlocksPlane(mask, nblock)

    else if(length(dim(mask))==3)
        getBlocksCube(mask, nblock)
}

#' Fit the hidden Potts model using a Markov chain Monte Carlo algorithm.
#' 
#' @param y A vector of observed pixel data.
#' @param neighbors A matrix of all neighbors in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param priors A list of priors for the parameters of the model.
#' @param mh A list of options for the Metropolis-Hastings algorithm.
#' @param niter The number of iterations of the algorithm to perform.
#' @param nburn The number of iterations to discard as burn-in.
#' @param truth A matrix containing the ground truth for the pixel labels.
#' @return A matrix containing MCMC samples for the parameters of the Potts model.
#' @export
mcmcPotts <- function(y, neighbors, blocks, priors, mh, niter=55000, nburn=5000, truth=NULL) {
  result <- .Call( "mcmcPotts", y, neighbors, blocks, niter, nburn, priors, mh, truth, PACKAGE = "bayesImageS")
}

#' Simulate pixel labels using chequerboard Gibbs sampling.
#' 
#' @param beta The inverse temperature parameter of the Potts model.
#' @param k The number of unique labels.
#' @param neighbors A matrix of all neighbors in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param niter The number of iterations of the algorithm to perform.
#' @param random Whether to initialize the labels using random or deterministic starting values.
#' @return A list containing the following elements:
#' \describe{
#'   \item{\code{alloc}}{An n by k matrix containing the number of times that pixel i was allocated to label j.}
#'   \item{\code{z}}{An \code{(n+1)} by k matrix containing the final sample from the Potts model after niter iterations of chequerboard Gibbs.}
#'   \item{\code{sum}}{An \code{niter} by 1 matrix containing the sum of like neighbors, i.e. the sufficient statistic of the Potts model, at each iteration.}
#'   }
#' @export
#' @examples
#' # Swendsen-Wang for a 2x2 lattice
#' neigh <- matrix(c(5,2,5,3,  1,5,5,4,  5,4,1,5,  3,5,2,5), nrow=4, ncol=4, byrow=TRUE)
#' blocks <- list(c(1,4), c(2,3))
#' res.Gibbs <- mcmcPottsNoData(0.7, 3, neigh, blocks, niter=200)
#' res.Gibbs$z
#' res.Gibbs$sum[200]
mcmcPottsNoData <- function(beta, k, neighbors, blocks, niter=1000, random=TRUE) {
  result <- .Call( "mcmcPottsNoData", beta, k, neighbors, blocks, niter, random, PACKAGE = "bayesImageS")  
}

#' Simulate pixel labels using the Swendsen-Wang algorithm.
#' 
#' The algorithm of Swendsen & Wang (1987) forms clusters of neighbouring pixels,
#' then updates all of the labels within a cluster to the same value. When
#' simulating from the prior, such as a Potts model without an external field,
#' this algorithm is very efficient.
#'
#' @param beta The inverse temperature parameter of the Potts model.
#' @param k The number of unique labels.
#' @param neighbors A matrix of all neighbors in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param niter The number of iterations of the algorithm to perform.
#' @param random Whether to initialize the labels using random or deterministic starting values.
#' @return A list containing the following elements:
#' \describe{
#'   \item{\code{alloc}}{An n by k matrix containing the number of times that pixel i was allocated to label j.}
#'   \item{\code{z}}{An \code{(n+1)} by k matrix containing the final sample from the Potts model after niter iterations of Swendsen-Wang.}
#'   \item{\code{sum}}{An \code{niter} by 1 matrix containing the sum of like neighbors, i.e. the sufficient statistic of the Potts model, at each iteration.}
#'   }
#' @export
#' @references Swendsen, R. H. & Wang, J.-S. (1987) "Nonuniversal critical dynamics in Monte Carlo simulations" \emph{Physical Review Letters} \bold{58}(2), 86--88, DOI: \doi{10.1103/PhysRevLett.58.86}
#' @examples
#' # Swendsen-Wang for a 2x2 lattice
#' neigh <- matrix(c(5,2,5,3,  1,5,5,4,  5,4,1,5,  3,5,2,5), nrow=4, ncol=4, byrow=TRUE)
#' blocks <- list(c(1,4), c(2,3))
#' res.sw <- swNoData(0.7, 3, neigh, blocks, niter=200)
#' res.sw$z
#' res.sw$sum[200]
swNoData <- function(beta, k, neighbors, blocks, niter=1000, random=TRUE) {
  result <- .Call( "swNoData", beta, k, neighbors, blocks, niter, random, PACKAGE = "bayesImageS")  
}

#' Fit a mixture of Gaussians to the observed data.
#' 
#' @param y A vector of observed pixel data.
#' @param niter The number of iterations of the algorithm to perform.
#' @param nburn The number of iterations to discard as burn-in.
#' @param priors A list of priors for the parameters of the model.
#' @return A matrix containing MCMC samples for the parameters of the mixture model.
#' @export
gibbsGMM <- function(y, niter=1000, nburn=500, priors=NULL) {
  result <- .Call( "gibbsGMM", y, niter, nburn, priors, PACKAGE = "bayesImageS")
}

#' Fit a univariate normal (Gaussian) distribution to the observed data.
#' 
#' @param y A vector of observed pixel data.
#' @param niter The number of iterations of the algorithm to perform.
#' @param priors A list of priors for the parameters of the model.
#' @return A list containing MCMC samples for the mean and standard deviation.
#' @examples
#' y <- rnorm(100,mean=5,sd=2)
#' res.norm <- gibbsNorm(y, priors=list(mu=0, mu.sd=1e6, sigma=1e-3, sigma.nu=1e-3))
#' summary(res.norm$mu[501:1000])
#' summary(res.norm$sigma[501:1000])
#' @export
gibbsNorm <- function(y, niter=1000, priors=NULL) {
  result <- .Call( "gibbsNorm", y, niter, priors, PACKAGE = "bayesImageS")
}

#' Fit a hidden Potts model to the observed data, using a fixed value of beta.
#' 
#' @inheritParams mcmcPotts
#' @param labels A matrix of pixel labels. 
#' @param beta The inverse temperature parameter of the Potts model.
#' @param mu A vector of means for the mixture components.
#' @param sd A vector of standard deviations for the mixture components.
#' @return A matrix containing MCMC samples for the parameters of the Potts model.
#' @export
gibbsPotts <- function(y, labels, beta, mu, sd, neighbors, blocks, priors, niter=1) {
  result <- .Call( "gibbsPotts", y, labels, beta, mu, sd, neighbors, blocks, priors, niter, PACKAGE = "bayesImageS")
}

#' Calculate the sufficient statistic of the Potts model for the given labels.
#'
#' @param labels A matrix of pixel labels. 
#' @param neighbors A matrix of all neighbors in the lattice, one row per pixel.
#' @param blocks A list of pixel indices, dividing the lattice into independent blocks.
#' @param k The number of unique labels.
#' @return The sum of like neighbors.
#' @export
sufficientStat <- function(labels, neighbors, blocks, k) {
  result <- .Call( "sufficientStat", labels, neighbors, blocks, k, PACKAGE = "bayesImageS")  
}

#' Get Edges of a Graph
#' 
#' Obtain edges of a 1D, 2D, or 3D graph based on the
#' neighbourhood structure.
#'
#' @param mask a vector, matrix, or 3D array specifying vertices of a graph. Vertices of value 1 are within the graph and 0 are not.
#' @param neiStruc a scalar, vector of four components, or \eqn{3\times4}	matrix corresponding to 1D, 2D, or 3D graphs. It specifies the neighbourhood structure. See \code{getNeighbors} for details.
#'
#' @return A matrix of two columns with one edge per row.  The edges connecting
#' vertices and their corresponding first neighbours are listed first, and
#' then those corresponding to the second neighbours, and so on and so
#' forth.  The order of neighbours is the same as in \code{getNeighbors}.
#' @export
#'
#' @details There could be more than one way to define the same 3D neighbourhood
#' structure for a graph (see Example 4 for illustration). 
#' @keywords spatial
#' @references 
#'  Winkler, G. (2003)
#'  "Image Analysis, Random Fields and Markov Chain Monte Carlo Methods: A Mathematical Introduction" (2nd ed.)
#'  \emph{Springer-Verlag}
#'  
#'  Feng, D. (2008)
#'  "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification"
#'  \emph{Ph. D. Dissertation, The University of Iowa} 
#' @usage getEdges(mask, neiStruc)
#' @examples
#'   #Example 1: get all edges of a 1D graph. 
#'   mask <- c(0,0,rep(1,4),0,1,1,0,0)
#'   getEdges(mask, neiStruc=2)
#'   
#'   #Example 2: get all edges of a 2D graph based on neighbourhood structure
#'   #           corresponding to the first-order Markov random field.
#'   mask <- matrix(1 ,nrow=2, ncol=3)
#'   getEdges(mask, neiStruc=c(2,2,0,0))
#'   
#'   #Example 3: get all edges of a 2D graph based on neighbourhood structure
#'   #           corresponding to the second-order Markov random field.
#'   mask <- matrix(1 ,nrow=3, ncol=3)
#'   getEdges(mask, neiStruc=c(2,2,2,2))
#'   
#'   #Example 4: get all edges of a 3D graph based on 6 neighbours structure
#'   #           where the neighbours of a vertex comprise its available
#'   #           N,S,E,W, upper and lower adjacencies. To achieve it, there
#'   #           are several ways, including the two below.
#'   mask <- array(1, dim=rep(3,3))
#'   n61 <- matrix(c(2,2,0,0,
#'                   0,2,0,0,
#'                   0,0,0,0), nrow=3, byrow=TRUE)
#'   n62 <- matrix(c(2,0,0,0,
#'                   0,2,0,0,
#'                   2,0,0,0), nrow=3, byrow=TRUE)
#'   e1 <- getEdges(mask, neiStruc=n61)
#'   e2 <- getEdges(mask, neiStruc=n62)
#'   e1 <- e1[order(e1[,1], e1[,2]),]
#'   e2 <- e2[order(e2[,1], e2[,2]),]
#'   all(e1==e2)
#'   
#'   #Example 5: get all edges of a 3D graph based on 18 neighbours structure
#'   #           where the neighbours of a vertex comprise its available
#'   #           adjacencies sharing the same edges or faces.
#'   #           To achieve it, there are several ways, including the one below.
#'   
#'   n18 <- matrix(c(2,2,2,2,
#'                   0,2,2,2,
#'                   0,0,2,2), nrow=3, byrow=TRUE)  
#'   mask <- array(1, dim=rep(3,3))
#'   getEdges(mask, neiStruc=n18)
getEdges <- function(mask, neiStruc){
    neighbors <- getNeighbors(mask, neiStruc)
    nvertex <- nrow(neighbors)
    nneighbor <- ncol(neighbors)
    edges <- do.call(rbind, lapply(1:nneighbor, function(i){
        edges <- cbind(1:nvertex, neighbors[,i])
        edges[edges[,1] < edges[,2],]
        }))
    edges <- edges[edges[,2] != nvertex+1, ]
    edges
}

pre <- function(mask){
    n <- sum(mask==1)
    if(is.vector(mask)){
        mask <- c(0, mask, 0)
        mask.new <- cumsum(mask)
        mask.new[mask==0] <- n + 1
    }
    else if(is.matrix(mask)){
        nr <- nrow(mask)
        nc <- ncol(mask)
        mask.new0 <- matrix(0, nr + 2, nc + 2)
        mask.new0[2:(nr+1), 2:(nc+1)] <- mask
        mask.new <- cumsum(mask.new0)
        mask.new[mask.new0==0] <- n + 1
    }
    else{
        nr <- dim(mask)[1]
        nc <- dim(mask)[2]
        nz <- dim(mask)[3]
        mask.new0 <- array(0, dim=c(nr + 2, nc + 2, nz + 2))
        mask.new0[2:(nr+1), 2:(nc+1), 2:(nz + 1)] <- mask
        mask.new <- cumsum(mask.new0)
        mask.new[mask.new0==0] <- n + 1
    }
    list(mask=mask.new, n=n)
}

get2NeighborsLine <- function(mask, n){
    focus <- which(mask <= n)
    cbind(mask[focus-1], mask[focus+1])
}

get2NeighborsPlane <- function(mask, n, nr, neiStruc){
    neighbors <- vector("list", 4)
    focus <- which(mask <= n)
    if(neiStruc[1]==T)
        neighbors[[1]] <- cbind(mask[focus-1], mask[focus+1])
    if(neiStruc[2]==T)
        neighbors[[2]] <- cbind(mask[focus-nr], mask[focus+nr])
    if(neiStruc[3]==T)
        neighbors[[3]] <- cbind(mask[focus-nr-1], mask[focus+nr+1])
    if(neiStruc[4]==T)
        neighbors[[4]] <- cbind(mask[focus-nr+1], mask[focus+nr-1])
    neighbors
}

get2NeighborsCube <- function(mask, n, nr, nc, neiStruc){
    neighbors <- vector("list", 12)
    focus <- which(mask <= n)
    n12 <- nr*nc
    if(neiStruc[1]==T)
        neighbors[[1]] <- cbind(mask[focus-1], mask[focus+1])
    if(neiStruc[2]==T)
        neighbors[[2]] <- cbind(mask[focus-nr], mask[focus+nr])
    if(neiStruc[3]==T)
        neighbors[[3]] <- cbind(mask[focus-nr-1], mask[focus+nr+1])
    if(neiStruc[4]==T)
        neighbors[[4]] <- cbind(mask[focus-nr+1], mask[focus+nr-1])

    if(neiStruc[5]==T)
        neighbors[[5]] <- cbind(mask[focus-1], mask[focus+1])
    if(neiStruc[6]==T)
        neighbors[[6]] <- cbind(mask[focus-n12], mask[focus+n12])
    if(neiStruc[7]==T)
        neighbors[[7]] <- cbind(mask[focus-n12-1], mask[focus+n12+1])
    if(neiStruc[8]==T)
        neighbors[[8]] <- cbind(mask[focus-n12+1], mask[focus+n12-1])

    if(neiStruc[9]==T)
        neighbors[[9]] <- cbind(mask[focus-nr], mask[focus+nr])
    if(neiStruc[10]==T)
        neighbors[[10]] <- cbind(mask[focus-n12], mask[focus+n12])
    if(neiStruc[11]==T)
        neighbors[[11]] <- cbind(mask[focus-n12-nr], mask[focus+n12+nr])
    if(neiStruc[12]==T)
        neighbors[[12]] <- cbind(mask[focus-n12+nr], mask[focus+n12-nr])


    neighbors
}

get2NeighborsCube3DDiad <- function(mask){
    expand <- matrix(c(-1, -1, -1,
                       -1, +1, +1,
                       -1, +1, -1,
                       -1, -1, +1,
                       +1, -1, -1,
                       +1, +1, +1,
                       +1, +1, -1,
                       +1, -1, +1), ncol=8)
    browser()
    n <- sum(mask)
    maskn <- array(cumsum(mask), dim=dim(mask))
    maskn[mask==0] <- n + 1
    focus <- which(mask==1, arr.ind=TRUE)
    focus <- t(focus)
    neighbors <- do.call(cbind, lapply(1:8, function(i)
                         maskn[t(focus + expand[,i])])) 
    
}

getNeighborsB <- function(neiStruc, neighbors2){
    nvertex <- nrow(neighbors2)
    neighbors <- matrix(nvertex+1, nrow=nvertex+1, ncol=neiStruc)
    neighbors[1:nvertex,1:2] <- neighbors2
    if(neiStruc > 2)
        for(i in 1:(neiStruc/2-1)){
            neighbors[,i*2+1] <- neighbors[neighbors[,(i-1)*2+1],1]
            neighbors[,i*2+2] <- neighbors[neighbors[,(i-1)*2+2],2]
        }
    neighbors[-(nvertex+1),]
}

getNeighborsLine <- function(mask, neiStruc){
    r <- pre(mask)
    neighbors2 <- get2NeighborsLine(r$mask, r$n)
    getNeighborsB(neiStruc, neighbors2)

}

getNeighborsPlane <- function(mask, neiStruc){
    r <- pre(mask)
    nr <- nrow(mask) + 2
    neighbors2 <- get2NeighborsPlane(r$mask, r$n, nr, neiStruc > 0)
    if(all(neiStruc <= 2))
        neighbors <- do.call(cbind, neighbors2)
    else{
        neighbors <- NULL
        for(i in 1:4){
            if(!is.null(neighbors2[[i]])){
                new <- getNeighborsB(neiStruc[i], neighbors2[[i]])
                neighbors <- cbind(neighbors, new)
            }
        }
    }
    neighbors
}

getNeighborsCube <- function(mask, neiStruc){
    r <- pre(mask)
    nr <- dim(mask)[1] + 2
    nc <- dim(mask)[2] + 2
    neiStruc <- as.vector(t(neiStruc))
    neighbors2 <- get2NeighborsCube(r$mask, r$n, nr, nc, neiStruc > 0)
    if(all(neiStruc <= 2))
        neighbors <- do.call(cbind, neighbors2)
    else{
        neighbors <- NULL
        for(i in 1:12){
            if(!is.null(neighbors2[[i]])){
                new <- getNeighborsB(neiStruc[i], neighbors2[[i]])
                neighbors <- cbind(neighbors, new)
            }
        }
    }
    neighbors
}


#' Get Neighbours of All Vertices of a Graph
#' 
#' Obtain neighbours of vertices of a 1D, 2D, or 3D graph.
#'
#' @param mask a vector, matrix, or 3D array specifying vertices within a graph. Vertices of value 1 are within the graph and 0 are not.
#' @param neiStruc a scalar, vector of four components, or \eqn{3\times4} matrix
#' corresponding to 1D, 2D, or 3D graphs. It gives the definition of
#' neighbours of a graph.
#' All components of \code{neiStruc} should be positive (\eqn{\ge 0})
#' even numbers.
#' For 1D graphs, \code{neiStruc} gives
#' the number of neighbours of each vertex.
#' For 2D graphs, \code{neiStruc}[1] specifies
#' the number of neighbours on vertical direction, \code{neiStruc}[2]
#' horizontal direction, \code{neiStruc}[3] north-west (NW) to south-east (SE)
#' diagonal direction, and \code{neiStruc}[4] south-west (SW) to
#' north-east (NE) diagonal direction. For 3D
#' graphs, the first row of \code{neiStruc} specifies the number of neighbours on
#' vertical direction, horizontal direction and two diagonal directions from
#' the 1-2 perspective, the second row the 1-3 perspective, and the
#' third row the 2-3 perspective. The index to
#' perspectives is represented with the leftmost subscript of the
#' array being the smallest.
#'
#' @return   A matrix with each row giving the neighbours of a vertex.
#' The number of the rows is equal to the number of 
#' vertices within the graph and the number or columns
#' is the number of neighbours of each vertex.
#' 
#' For a 1D graph, if each vertex has two neighbours,
#' The first column are the neighbours on the left-hand side of
#' corresponding vertices and the second column the right-hand side.
#' For the vertices on boundaries, missing neighbours are represented by
#' the number of vertices within a graph plus 1.
#' When \code{neiStruc} is bigger than 2, The first two columns
#' are the same as when \code{neiStruc} is equal to 2; the third column
#' are the neighbours on the left-hand side of the vertices on the first column;
#' the forth column are the neighbours on the right-hand side of the vertices
#' on the second column, and so on and so forth. And again for the
#' vertices on boundaries, their missing neighbours are represented by
#' the number of vertices within a graph plus 1.
#' 
#' For a 2D graph, the index to vertices is column-wised. For each
#' vertex, the order of neighbours are as follows. First are those
#' on the vertical direction, second the horizontal
#' direction, third the NW to SE diagonal
#' direction, and forth the SW to NE diagonal
#' direction. For each direction, the neighbours of every vertex
#' are arranged in the same way as in a 1D graph.
#' 
#' For a 3D graph, the index to vertices is that
#' the leftmost subscript of the array moves the fastest.  For each
#' vertex, the neighbours from the 1-2 perspective
#' appear first and then the 1-3 perspective and finally
#' the 2-3 perspective. For each perspective, the neighbours are arranged
#' in the same way as in a 2D graph.
#'
#' @export
#' @details   There could be more than one way to define the same 3D neighbourhood
#' structure for a graph (see Example 3 for illustration). 
#' @keywords spatial
#' @references 
#'  Winkler, G. (2003)
#'  "Image Analysis, Random Fields and Markov Chain Monte Carlo Methods: A Mathematical Introduction" (2nd ed.)
#'  \emph{Springer-Verlag}
#'  
#'  Feng, D. (2008)
#'  "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification"
#'  \emph{Ph. D. Dissertation, The University of Iowa} 
#' @usage getNeighbors(mask, neiStruc)
#' @examples
#'   #Example 1: get all neighbours of a 1D graph.
#'   mask <- c(0,0,rep(1,4),0,1,1,0,0,1,1,1)
#'   getNeighbors(mask, neiStruc=2)
#'   
#'   #Example 2: get all neighbours of a 2D graph based on neighbourhood structure
#'   #           corresponding to the second-order Markov random field.
#'   mask <- matrix(1, nrow=2, ncol=3)
#'   getNeighbors(mask, neiStruc=c(2,2,2,2))
#'   
#'   #Example 3: get all neighbours of a 3D graph based on 6 neighbours structure
#'   #           where the neighbours of a vertex comprise its available
#'   #           N,S,E,W, upper and lower adjacencies. To achieve it, there
#'   #           are several ways, including the two below.
#'   mask <- array(1, dim=rep(3,3))
#'   n61 <- matrix(c(2,2,0,0,
#'                   0,2,0,0,
#'                   0,0,0,0), nrow=3, byrow=TRUE)
#'   n62 <- matrix(c(2,0,0,0,
#'                   0,2,0,0,
#'                   2,0,0,0), nrow=3, byrow=TRUE)
#'   n1 <- getNeighbors(mask, neiStruc=n61)
#'   n2 <- getNeighbors(mask, neiStruc=n62)
#'   n1 <- apply(n1, 1, sort)
#'   n2 <- apply(n2, 1, sort)
#'   all(n1==n2)
#'   
#'   #Example 4: get all neighbours of a 3D graph based on 18 neighbours structure
#'   #           where the neighbours of a vertex comprise its available
#'   #           adjacencies sharing the same edges or faces.
#'   #           To achieve it, there are several ways, including the one below.
#'   
#'   n18 <- matrix(c(2,2,2,2,
#'                   0,2,2,2,
#'                   0,0,2,2), nrow=3, byrow=TRUE)  
#'   mask <- array(1, dim=rep(3,3))
#'   getNeighbors(mask, neiStruc=n18)
getNeighbors <- function(mask, neiStruc){
    if(!(is.vector(mask) || is.matrix(mask) || length(dim(mask))==3))
        stop("The graph has to be in 1D, 2D, or 3D.")

    if(length(mask) < 3)
        stop("There are at least 3 vertices.")
    if(is.vector(mask))
        if(!is.vector(neiStruc) || length(neiStruc) != 1)
            stop("For a line, you need to tell the number of neighbors
                 on only one direction.")
    if(is.matrix(mask))
        if(!is.vector(neiStruc) || length(neiStruc)!=4)
            stop("For a plane, you need to tell the number of neighbors
                 on four directions.")
    if(length(dim(mask))==3)
        if(!is.matrix(neiStruc) || nrow(neiStruc)!=3 || ncol(neiStruc)!=4)
            stop("For a cube, you need to tell the number of neighbors
                 on four directions from all three perspective.")

    if(! all(neiStruc >= 0))
        stop("Number of neighbors have to be >= 0.")
    if(! all(neiStruc %% 2 == 0))
        stop("Number of neighbors have to be all even.")

    if(is.vector(mask))
        neighbors <- getNeighborsLine(mask, neiStruc)

    if(is.matrix(mask))
        neighbors <- getNeighborsPlane(mask, neiStruc)

    if(length(dim(mask))==3)
        neighbors <- getNeighborsCube(mask, neiStruc)

    neighbors
}

---
title: "mcmcPotts"
author: "Matt Moores"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{mcmcPotts}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteKeyword{Gibbs sampler}
  %\VignetteKeyword{Coupling From The Past}
---
This post looks at the convergence of the chequerboard Gibbs sampler for the hidden Potts model, in the presence of an external field. This algorithm is implemented as the function [`mcmcPotts`](https://mooresm.github.io/bayesImageS/reference/mcmcPotts.html) in my R package, [**bayesImageS**](https://CRAN.R-project.org/package=bayesImageS). Previous posts have looked at the convergence of Gibbs and Swendsen-Wang algorithms without an external field, as implemented in [`mcmcPottsNoData`](https://mattstats.wordpress.com/2017/01/17/bayesimagesmcmcpottsnodata/) and [`swNoData`](https://mattstats.wordpress.com/2013/12/13/swnodata/) functions. 

------
The most accurate way to measure convergence is using the coupling time of a perfect sampling algorithm, such as [coupling from the past](https://mattstats.wordpress.com/2017/11/01/coupling-from-the-past/) (CFTP). However, we can obtain a rough estimate by monitoring the distribution of the sufficient statistic:
$$
\text{S}(\mathbf{z}) = \sum_{\{i,j\} \in \mathcal{E}} \delta(z_i, z_j)
$$
Where $\delta(x,y)$ is the [Kronecker delta](https://en.wikipedia.org/wiki/Kronecker_delta) function. Note that this sum is defined over the *unique* undirected edges of the lattice, to avoid double-counting. Under this definition, the critical temperature of the $q$-state Potts model is $\log\{1 + \sqrt{q}\}$, or $\approx 0.88$ for the Ising model with $q=2$ unique labels. Some papers state that the critical temperature of the Ising model is $0.44$, but this is because they have used a different definition of $\text{S}(\mathbf{z})$.

We will generate synthetic data for a sequence of values of the inverse temperature, $\beta = (0.22, 0.44, 0.88, 1.32)$:
```{r eval=FALSE}
library(bayesImageS)
library(doParallel)
set.seed(123)
q <- 2
beta <- c(0.22, 0.44, 0.88, 1.32)
mask <- matrix(1,nrow=500,ncol=500)
n <- prod(dim(mask))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
maxS <- nrow(edges)

cl <- makeCluster(min(4, detectCores()))
registerDoParallel(cl)

tm <- system.time(synth <- foreach (i=1:length(beta),
                      .packages="bayesImageS") %dopar% {
  gen <- list()
  gen$beta <- beta[i]
  # generate labels
  sw <- swNoData(beta[i], q, neigh, block, 200)
  gen$z <- sw$z
  gen$sum <- sw$sum[200]
  # now add noise
  gen$mu <- rnorm(2, c(-1,1), 0.5)
  gen$sd <- 1/sqrt(rgamma(2, 1.5, 2))
  gen$y <- rnorm(n, gen$mu[(gen$z[1:n,1])+1],
                    gen$sd[(gen$z[1:n,1])+1])
  gen
})
stopCluster(cl)
```

```{r echo=FALSE}
library(bayesImageS)
data("synth", package = "bayesImageS")
print(synth$tm)
```


Now let's look at the distribution of Gibbs samples for the first dataset, using a fixed value of $\beta$:
```{r eval=FALSE}
priors <- list()
priors$k <- q
priors$mu <- c(-1,1)
priors$mu.sd <- rep(0.5,q)
priors$sigma <- rep(2,q)
priors$sigma.nu <- rep(1.5,q)
priors$beta <- rep(synth[[1]]$beta, 2)

mh <- list(algorithm="ex", bandwidth=1, adaptive=NA,
           auxiliary=1)
tm <- system.time(res <- mcmcPotts(synth[[1]]$y, neigh,
                            block, priors, mh, 100, 50))
```
```{r}
data("res", package = "bayesImageS")
print(res$tm)
mean(res$sum[51:100])
print(synth[[1]]$sum)
ts.plot(res$sum, xlab="MCMC iterations", ylab=expression(S(z)))
abline(h=synth[[1]]$sum, col=4, lty=2)
```

As expected for $\beta=0.22$ with $n=400$ pixels, convergence takes only a dozen iterations or so. The same is true for $\beta=0.66$:
```{r eval=FALSE}
priors$beta <- rep(synth[[2]]$beta, 2)
tm2 <- system.time(res2 <- mcmcPotts(synth[[2]]$y,
                      neigh, block, priors, mh, 100, 50))
```
```{r echo=FALSE}
data("res2", package = "bayesImageS")
print(res2$tm)
ts.plot(res2$sum, xlab="MCMC iterations", ylab=expression(S(z)))
abline(h=synth[[2]]$sum, col=4, lty=2)
```

Now with $\beta=0.88$:
```{r eval=FALSE}
priors$beta <- rep(synth[[3]]$beta, 2)
tm3 <- system.time(res3 <- mcmcPotts(synth[[3]]$y,
                      neigh, block, priors, mh, 100, 50))
```

```{r echo=FALSE}
data("res3", package = "bayesImageS")
print(res3$tm)
ts.plot(res3$sum, xlab="MCMC iterations", ylab=expression(S(z)))
abline(h=synth[[3]]$sum, col=4, lty=2)
```

So far, so good. Now let's try with $\beta=1.32$:
```{r eval=FALSE}
priors$beta <- rep(synth[[4]]$beta, 2)
tm4 <- system.time(res4 <- mcmcPotts(synth[[4]]$y,
                    neigh, block, priors, mh, 100, 50))
```

```{r echo=FALSE}
data("res4", package = "bayesImageS")
print(res4$tm)
ts.plot(res4$sum, xlab="MCMC iterations", ylab=expression(S(z)))
abline(h=synth[[4]]$sum, col=4, lty=2)
```

This doesn't really count as slow mixing, since the Gibbs sampler has converged within 50 iterations for a lattice with $500 \times 500$ pixels. Compare how long it takes without the external field:
```{r eval=FALSE}
tm5 <- system.time(res5 <- mcmcPottsNoData(synth[[4]]$beta, q,
                                    neigh, block, 5000))
```
```{r echo=FALSE}
data("res5", package = "bayesImageS")
print(res5$tm)
ts.plot(res5$sum, xlab="MCMC iterations", ylab=expression(S(z)))
abline(h=synth[[4]]$sum, col=4, lty=2)
```

This explains why single-site Gibbs sampling should **never** be used for the auxiliary iterations in ABC or the exchange algorithm, but it is usually fine to use when updating the hidden labels. Note that all of these results have been for a fixed $\beta$. It is more difficult to assess convergence when $\beta$ is unknown. A topic for a future post!

---
title: "mcmcPottsNoData"
author: "Matt Moores"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{mcmcPottsNoData}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteKeyword{Gibbs sampler}
  %\VignetteKeyword{Swendsen-Wang}
---

```{r setup, include=FALSE}
library(bayesImageS)
set.seed(12)
```

This is a follow up to [my previous post](https://mattstats.wordpress.com/2013/12/13/swnodata/) about the Swendsen-Wang (SW) algorithm, where I mentioned that SW has better convergence properties than Gibbs when the inverse temperature parameter $\beta$ is large. This difference can be quantified by initialising the two algorithms at known starting points and measuring how many iterations it takes to converge. This is the second in a series of posts describing the functions and algorithms that I have implemented in the R package **bayesImageS**, which is [now available on CRAN](https://CRAN.R-project.org/package=bayesImageS).

------
The Potts model has a doubly-intractable likelihood, so its expectation and variance cannot be computed exactly. Instead, we can use Markov chain Monte Carlo (MCMC) algorithms such as SW or Gibbs sampling to simulate from its distribution for a given value of $\beta$. However, we need to know how many MCMC iterations to use, so that the chain will have converged to a steady state. Otherwise, any inference using the MCMC samples will be biased.

In the following, the labels **z** of the Potts model can take *k* different values. This state space is not ordered, so algorithms such as perfect sampling (Propp & Wilson, 1996; Huber, 2016) cannot be applied. The Potts model is a member of the exponential family, so it has a sufficient statistic S(**z**) which is the count of like neighbours. The maximum value of S(**z**), which we will call $M$, is equal to $2(n - \sqrt{n})$ for a regular, square lattice. For example, $M = 112$ for an 8x8 lattice; $M = 31,000$ for 125x125; and $M = 1,998,000$ for 1000x1000.

There are two exceptions where the distribution of the Potts model can be computed exactly. When $\beta=0$ the labels z are independent, hence the sufficient statistic S(**z**) follows a Binomial distribution with expectation $M/k$ and variance $M(1/k)(1 - 1/k)$. For an 8x8 lattice with k=3, the expectation is `64/3` with a variance of 24.89. As β approaches infinity, all of the labels have the same value almost surely. This means that the expectation approaches M asymptotically, while the variance approaches 0.

We can use the endpoints of the distribution to estimate how long the SW and Gibbs algorithms take to converge. The algorithm is initialised at one endpoint, then we monitor S(**z**) at each iteration until the distribution of the samples has converged to the known expectation and variance. First, let's look at chequerboard Gibbs sampling for an 8×8 lattice with *k*=3:

```{r fig.cap="Chequerboard Gibbs sampling for a regular 8x8 lattice with k=3 labels."}
k <- 3
n <- 8*8
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.Gibbs <- mcmcPottsNoData(beta=5, k=3, neigh, block, niter=100)
ts.plot(res.Gibbs$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.Gibbs$sum[51:100])
var(res.Gibbs$sum[51:100])
```

We can see that it only takes around 35 iterations for the Gibbs sampler to converge for a lattice of that size. Now for a 125×125 lattice:
```{r fig.cap="Chequerboard Gibbs sampling for a regular 125x125 lattice with k=3 labels."}
n <- 125*125
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.Gibbs <- mcmcPottsNoData(beta=5, k=3, neigh, block, niter=2000)
ts.plot(res.Gibbs$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.Gibbs$sum[1001:2000])
var(res.Gibbs$sum[1001:2000])
```

Even after 2000 iterations, the distribution of S(**z**) might not have converged to the known value. Now let's see how Swendsen-Wang performs for the same lattice:
```{r}
res.SW <- swNoData(beta=5, k=3, neigh, block, niter=50)
ts.plot(res.SW$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.SW$sum[26:50])
var(res.SW$sum[26:50])
```

After 25 iterations, SW has already converged to the exact distribution. Even though this algorithm is much more expensive for each iteration, it more than makes up for that in efficiency when $\beta$ is large.

```{r}
n <- 1000*1000
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
print(paste(sum(mask),"pixels"))
print(paste("maximum sufficient statistic S(z) =",nrow(edges)))
res.SW <- swNoData(beta=5, k=3, neigh, block, niter=100)
ts.plot(res.SW$sum, ylim=c(nrow(edges)/3, nrow(edges)))
abline(h=nrow(edges), col=2, lty=3)
summary(res.SW$sum[81:100])
var(res.SW$sum[81:100])
```

Now let's see what happens when we go in the other direction: initialising the lattice with all labels set to the same value, then updating with $\beta=0$:

```{r}
n <- 125*125
mask <- matrix(1,nrow=sqrt(n),ncol=sqrt(n))
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))
system.time(res2.Gibbs <- mcmcPottsNoData(beta=0, k=3, neigh, block, niter=100, random=FALSE))
ts.plot(res2.Gibbs$sum, ylim=range(c(res2.Gibbs$sum, nrow(edges))))
abline(h=nrow(edges), col=2, lty=3)
abline(h=nrow(edges)/3, col=4, lty=3)
summary(res2.Gibbs$sum[51:100])
var(res2.Gibbs$sum[51:100])
```

The distribution of all 500 samples are very close to the exact distribution with mean 10333.33 and variance 6888.89:
```{r}
muSz <- nrow(edges)/3
sdSz <- sqrt(nrow(edges)*(1/3)*(2/3))
hist(res2.Gibbs$sum[51:100], freq=FALSE, breaks=20, col=3,
     xlim=range(c(res2.Gibbs$sum,muSz - 3*sdSz, muSz + 3*sdSz)))
abline(v=nrow(edges)/3, col=4, lty=3, lwd=3)
curve(dnorm(x, mean=nrow(edges)/3, sd=sqrt(nrow(edges)*(1/3)*(2/3))), 
          col="darkblue", lwd=2, lty=2, add=TRUE, yaxt="n")
```

Now for Swendsen-Wang:
```{r}
system.time(res2.SW <- swNoData(beta=0, k=3, neigh, block, niter=100, random=FALSE))
ts.plot(res2.SW$sum, ylim=range(c(res2.SW$sum, nrow(edges))))
abline(h=nrow(edges), col=2, lty=3)
abline(h=nrow(edges)/3, col=4, lty=3)
summary(res2.SW$sum[51:100])
var(res2.SW$sum[51:100])
```

```{r}
hist(res2.SW$sum[51:100], freq=FALSE, breaks=20, col=3,
     xlim=range(c(res2.Gibbs$sum,muSz - 3*sdSz, muSz + 3*sdSz)))
abline(v=nrow(edges)/3, col=4, lty=3, lwd=3)
curve(dnorm(x, mean=muSz, sd=sdSz), 
          col="darkblue", lwd=2, lty=2, add=TRUE, yaxt="n")
```

---
title: "swNoData"
author: "Matt Moores"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
bibliography: refs.bib
vignette: >
  %\VignetteIndexEntry{swNoData}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteKeyword{Swendsen-Wang}
  %\VignetteKeyword{Gibbs sampling}
---

Gibbs sampling was originally designed by @Geman1984 for drawing updates from the Gibbs distribution, hence the name. However, single-site Gibbs sampling exhibits poor mixing due to the posterior correlation between the pixel labels. Thus it is very slow to converge when the correlation (controlled by the inverse temperature $\beta$) is high.

The algorithm of @Swendsen1987 addresses this problem by forming clusters of neighbouring pixels, then updating all of the labels within a cluster to the same value. When simulating from the prior, such as a Potts model without an external field, this algorithm is very efficient.

------
The `SW` function in the **PottsUtils** package is implemented in a combination of `R` and `C`. The `swNoData` function in **bayesImageS** is implemented using **RcppArmadillo**, which gives it a speed advantage. It is worth noting that the intention of **bayesImageS** is not to replace **PottsUtils**. Rather, an efficient Swendsen-Wang algorithm is used as a building block for implementations of ABC [@Grelaud2009], path sampling [@Gelman1998], and the exchange algorithm [@Murray2006]. These other algorithms will be covered in future posts.

There are two things that we want to keep track of in this simulation study: the speed of the algorithm and the distribution of the summary statistic. We will be using `system.time(..)` to measure both CPU and elapsed (wall clock) time taken for the same number of iterations, for a range of inverse temperatures:
```{r}
beta <- seq(0,2,by=0.1)
tmMx.PU <- tmMx.bIS <- matrix(nrow=length(beta),ncol=2)
rownames(tmMx.PU) <- rownames(tmMx.bIS) <- beta
colnames(tmMx.PU) <- colnames(tmMx.bIS) <- c("user","elapsed")
```

We will discard the first 100 iterations as burn-in and keep the remaining 500.
```{r}
iter <- 600
burn <- 100
samp.PU <- samp.bIS <- matrix(nrow=length(beta),ncol=iter-burn)
```

The distribution of pixel labels can be summarised by the sufficient statistic of the Potts model:

$$
S(z) = \sum_{i \sim \ell \in \mathscr{N}} \delta(z_i, z_\ell)
$$

where $i \sim \ell \in \mathscr{N}$ are all of the pairs of neighbours in the lattice (ie. the cliques) and $\delta(u,v)$ is 1 if $u = v$ and 0 otherwise (the Kronecker delta function). `swNoData` returns this automatically, but with `SW` we will need to use the function `sufficientStat` to calculate the sufficient statistic for the labels.

```{r}
library(bayesImageS)

mask <- matrix(1,50,50)
neigh <- getNeighbors(mask, c(2,2,0,0))
block <- getBlocks(mask, 2)
edges <- getEdges(mask, c(2,2,0,0))

n <- sum(mask)
k <- 2
bcrit <- log(1 + sqrt(k))
maxSS <- nrow(edges)

for (i in 1:length(beta)) {
  if (requireNamespace("PottsUtils", quietly = TRUE)) {
    tm <- system.time(result <- PottsUtils::SW(iter,n,k,edges,beta=beta[i]))
    tmMx.PU[i,"user"] <- tm["user.self"]
    tmMx.PU[i,"elapsed"] <- tm["elapsed"]
    res <- sufficientStat(result, neigh, block, k)
    samp.PU[i,] <- res$sum[(burn+1):iter]
    print(paste("PottsUtils::SW",beta[i],tm["elapsed"],median(samp.PU[i,])))
   } else {
      print("PottsUtils::SW unavailable on this platform.")
   }

  
  # bayesImageS
  tm <- system.time(result <- swNoData(beta[i],k,neigh,block,iter))
  tmMx.bIS[i,"user"] <- tm["user.self"]
  tmMx.bIS[i,"elapsed"] <- tm["elapsed"]
  samp.bIS[i,] <- result$sum[(burn+1):iter]
  print(paste("bayesImageS::swNoData",beta[i],tm["elapsed"],median(samp.bIS[i,])))
}
```

Here is the comparison of elapsed times between the two algorithms (in seconds):
```{r}
summary(tmMx.PU)
summary(tmMx.bIS)
boxplot(tmMx.PU[,"elapsed"],tmMx.bIS[,"elapsed"],ylab="seconds elapsed",names=c("SW","swNoData"))
```

On average, `swNoData` using **RcppArmadillo** [@Eddelbuettel2013] is seven times faster than `SW`.

```{r}
s_z <- c(samp.PU,samp.bIS)
s_x <- rep(beta,times=iter-burn)
s_a <- rep(1:2,each=length(beta)*(iter-burn))
s.frame <- data.frame(s_z,c(s_x,s_x),s_a)
names(s.frame) <- c("stat","beta","alg")
s.frame$alg <- factor(s_a,labels=c("SW","swNoData"))
  if (requireNamespace("lattice", quietly = TRUE)) {
    lattice::xyplot(stat ~ beta | alg, data=s.frame)
  }
plot(c(s_x,s_x),s_z,pch=s_a,xlab=expression(beta),ylab=expression(S(z)))
abline(v=bcrit,col="red")
```

The overlap between the two distributions is almost complete, although it is a bit tricky to verify that statistically. The relationship between $beta$ and $S(z)$ is nonlinear and heteroskedastic.

```{r}
rowMeans(samp.bIS) - rowMeans(samp.PU)
apply(samp.PU, 1, sd)
apply(samp.bIS, 1, sd)
s.frame$beta <- factor(c(s_x,s_x))
  if (requireNamespace("PottsUtils", quietly = TRUE)) {
    s.fit <- aov(stat ~ alg + beta, data=s.frame)
    summary(s.fit)
    TukeyHSD(s.fit,which="alg")
  }
```

### References

---
title: "bayesImageS: An R Package for Bayesian Image Segmentation using a Hidden Potts Model"
author: "Matt Moores, Anthony N. Pettitt and Kerrie Mengersen"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
bibliography: refs.bib
vignette: >
  %\VignetteIndexEntry{Bayesian Methods for Image Segmentation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteKeyword{Approximate Bayesian Computation}
  %\VignetteKeyword{Composite likelihood}
  %\VignetteKeyword{Hidden Markov random field}
  %\VignetteKeyword{Image analysis}
  %\VignetteKeyword{Pseudo-marginal method}
  %\VignetteKeyword{Thermodynamic integration}
---

# Introduction

Image segmentation can be viewed as the task of labelling the observed pixels $\mathbf{y}$ according to a finite set of discrete states $\mathbf{z} \in \{ 1, \dots, k \}$. The hidden \citet{Potts1952} model allows for spatial correlation between neighbouring labels in the form of a Markov random field. The latent labels follow a Gibbs distribution, which is specified in terms of its conditional probabilities:
  \begin{equation}
  \label{eq:Potts}
  p(z_i | z_{\setminus i}, \beta) = \frac{\exp\left\{\beta\sum_{i \sim \ell}\delta(z_i,z_\ell)\right\}}{\sum_{j=1}^k \exp\left\{\beta\sum_{i \sim \ell}\delta(j,z_\ell)\right\}}
  \end{equation}
  where $\beta$ is the inverse temperature, $z_{\setminus i}$ represents all of the labels except $z_i$, $i \sim \ell$ are the  neighbouring pixels of $i$, and $\delta(u,v)$ is the Kronecker delta function. Thus, $\sum_{i \sim \ell}\delta(z_i,z_\ell)$ is a count of the neighbours that share the same label.

The observation equation links the latent labels to the corresponding pixel values:
  \begin{equation}
  \label{eq:obs}
  p(\mathbf{y} | \mathbf{z}, \boldsymbol\theta) = \prod_{i=1}^n p(y_i | z_i, \theta_{z_i})
  \end{equation}
  where $\theta_{j}$ are the parameters that govern the distribution of the pixel values with label $j$. The hidden Potts model can thus be viewed as a spatially-correlated generalisation of the finite mixture model \citep{Ryden1998}. We assume that the pixels with label $j$ share a common mean $\mu_j$ corrupted by additive Gaussian noise with variance $\sigma_j^2$:
  \begin{equation}
  \label{eq:obs2}
y_i | z_i=j, \mu_j, \sigma^2_j \;\sim\; \mathcal{N}\left( \mu_j, \sigma^2_j \right)
  \end{equation}
 
The Gibbs distribution is a member of the exponential family and so there is a sufficient statistic for this model, as noted by \citet{Grelaud2009}:
  \begin{equation}
  \label{eq:potts_stat}
\mathrm{S}(\mathbf{z}) = \sum_{i \sim \ell \in \mathcal{E}} \delta(z_i,z_\ell)
  \end{equation}
This statistic represents the total number of like neighbour pairs in the image. The likelihood $p(\mathbf{y},\mathbf{z} | \boldsymbol\theta, \beta)$ can therefore be factorised into $p(\mathbf{y} | \mathbf{z}, \boldsymbol\theta) p(\mathrm{S}(\mathbf{z}) | \beta)$, where the second factor does not depend on the observed data, but only on the sufficient statistic. The joint posterior is then:
\begin{equation}
\label{eq:joint_post}
p(\boldsymbol\theta, \beta, \mathbf{z} | \mathbf{y}) \propto p(\mathbf{y} | \mathbf{z}, \boldsymbol\theta) \pi(\boldsymbol\theta) p(\mathrm{S}(\mathbf{z}) | \beta) \pi(\beta)
\end{equation}
The conditional distributions $p(\boldsymbol\theta | \mathbf{z}, \mathbf{y})$ and $p(z_i | z_{\setminus i}, \beta, y_i, \boldsymbol\theta_{z_i})$ can be simulated using Gibbs sampling, but $p(\beta | \mathbf{y}, \mathbf{z}, \boldsymbol\theta)$ involves an intractable normalising constant $\mathcal{C}(\beta)$:
  \begin{eqnarray}
  \label{eq:beta_post}
  p(\beta \mid \mathbf{y}, \mathbf{z}, \boldsymbol\theta) &=&  \frac{p(\mathrm{S}(\mathbf{z}) | \beta) \pi(\beta)}{\int_\beta p(\mathrm{S}(\mathbf{z}) | \beta) \pi(d \beta)}
\\   \label{eq:beta}
 &\propto& \frac{\exp\left\{ \beta\, \mathrm{S}(\mathbf{z}) \right\}}{\mathcal{C}(\beta)} \pi(\beta)
  \end{eqnarray}
The normalising constant is also known as a partition function in statistical physics. It has computational complexity of $\mathcal{O}(n k^n)$, since it involves a sum over all possible combinations of the labels $\mathbf{z} \in \mathcal{Z}$:
  \begin{equation}
  \label{eq:norm}
\mathcal{C}(\beta) = \sum_{\mathbf{z} \in \mathcal{Z}} \exp\left\{\beta\, \mathrm{S}(\mathbf{z})\right\}
  \end{equation}
It is infeasible to calculate this value exactly for nontrivial images, thus computational approximations are required.

This paper describes the \proglang{R} package \pkg{bayesImageS}, which is available from the Comprehensive R Archive Network (CRAN) at  https://CRAN.R-project.org/package=bayesImageS. This package implements five major algorithms for intractable likelihoods in Bayesian image analysis. These methods provide alternative means to simulate parameter values from (\ref{eq:beta}) without computing the normalising constant. We describe the algorithms in terms of Markov chain Monte Carlo (MCMC) to enable direct comparison, although we also mention other approaches where applicable, such as particle-based (SMC and PMCMC) methods. Reference implementations of all of these methods are available from various sources described below, but for the purpose of comparison we have reimplemented the algorithms using \pkg{RcppArmadillo} \citep{Eddelbuettel2013}.

There are a number of contributed packages available for \proglang{R}, for example on CRAN, which provide image segmentation using Potts and other models: \pkg{potts}, \pkg{PottsUtils}, \pkg{mritc}, \pkg{bayess}

# Informative Priors

It can be useful to place informative priors on the parameters of the mixture components

The external field prior \citep{MHHM}

# Algorithms for Intractable Likelihoods

## Pseudolikelihood and Composite Likelihood

Pseudolikelihood is the simplest of the methods that we have implemented and also the fastest. \citet{Ryden1998} showed that the intractable distribution (\ref{eq:beta}) could be approximated using the product of the conditional densities given by (\ref{eq:Potts}):
 \begin{equation} \label{eq:pseudo}
p(\mathrm{S}(\mathbf{z}) | \beta) \approx \prod_{i=1}^n p(z_i | z_{\setminus i}, \beta)
\end{equation}
This enables updates for the inverse temperature at iteration $t$ to be simulated using a Metropolis-Hastings (M-H) step, with acceptance ratio:
\begin{equation} \label{mh:ratio}
\rho = \min\left( 1, \frac{p(\mathbf{z}|\beta') \pi(\beta') q(\beta_{t-1} | \beta')}{p(\mathbf{z}|\beta_{t-1}) \pi(\beta_{t-1}) q(\beta' | \beta_{t-1})} \right)
\end{equation}

The M-H proposal density $q(\beta'|\beta_{t-1})$ can be any distribution such that $\int q(\beta'|\beta_{t-1})\, d\beta' = 1$. However, there is a tradeoff between exploring the full state space and ensuring that the probability of acceptance is sufficiently high. We use the adaptive random walk (RWMH) algorithm of \citet{Garthwaite2010}, which automatically tunes the bandwidth of the proposal density to target a given M-H acceptance rate. When a symmetric proposal density is used, $q(\beta'|\beta_{t-1}) = q(\beta_{t-1}|\beta')$ and so this term cancels out in the M-H ratio \eqref{mh:ratio}. Likewise, under a uniform prior for the inverse temperature, $\pi(\beta') = \pi(\beta_{t-1}) = 1$. The natural logarithm of $\rho$ is used in practice to improve numerical stability. 

\begin{figure}
        \centering
        \begin{subfigure}{0.65\textwidth}
                \includegraphics[width=\textwidth]{pl_exp_n12k3.eps}
                \caption{Expectation.}
                \label{f:pl_exp}
        \end{subfigure}%
\qquad
        \begin{subfigure}{0.65\textwidth}
                \includegraphics[width=\textwidth]{pl_sd_n12k3.eps}
                \caption{Standard deviation.}
                \label{f:pl_sd}
        \end{subfigure}%
\caption{Approximation error of pseudolikelihood for $n=12,\,k=3$ in comparison to the exact likelihood calculated using a brute force method: (a) $\sum_{\mathbf{z} \in \mathcal{Z}} \mathrm{S}(\mathbf{z}) p(\mathrm{S}(\mathbf{z}) | \beta)$ using either Equation~(\ref{eq:beta}) or (\ref{eq:pseudo}); (b) $\sqrt{\sum_{\mathbf{z} \in \mathcal{Z}} \left( \mathrm{S}(\mathbf{z}) - \mathbb{E}_{\mathbf{z}|\beta}[\mathrm{S}(\mathbf{z})] \right)^2 p(\mathrm{S}(\mathbf{z}) | \beta)}$} 
\label{f:pl}
\end{figure}
Pseudolikelihood is exact when $\beta=0$ and provides a reasonable approximation for small values of the inverse temperature. However, the approximation error increases rapidly for $\beta \ge \beta_{crit}$, as illustrated by Figure \ref{f:pl}. This is due to long-range dependence between the labels, which is inadequately modelled by the local approximation. The implications of this inaccuracy for posterior inference will be demonstrated in Section \ref{s:results}.

\citet{Ryden1998} referred to Equation \eqref{eq:pseudo} as point pseudolikelihood, since the conditional distributions are computed for each pixel individually. They suggested that the accuracy could be improved using block pseudolikelihood. This is where the likelihood is calculated exactly for small blocks of pixels, then \eqref{eq:pseudo} is modified to be the product of the blocks:
\begin{equation}
p(\mathbf{z}|\beta) \approx \prod_{i=1}^{N_B} p(\mathbf{z}_{B_i} | \mathbf{z}_{\setminus B_i}, \beta)
\label{eq:pl_comp}
\end{equation}
where $N_B$ is the number of blocks, $\mathbf{z}_{B_i}$ are the labels of the pixels in block $B_i$, and $\mathbf{z}_{\setminus B_i}$ are all of the labels except for $\mathbf{z}_{B_i}$. This is a form of composite likelihood, where the likelihood function is approximated as a product of simplified factors \citep{Varin2011}. \citet{Friel2012} compared point pseudolikelihood to composite likelihood with blocks of $3 \times 3$, $4 \times 4$, $5 \times 5$, and $6 \times 6$ pixels. \citeauthor{Friel2012} showed that (\ref{eq:pl_comp}) outperformed (\ref{eq:pseudo}) for the Ising ($k=2$) model with $\beta < \beta_{crit}$. \citet{Okabayashi2011} discuss composite likelihood for the Potts model with $k > 2$ and have provided an open source implementation in the \proglang{R} package \pkg{potts}.

Evaluating the conditional likelihood in (\ref{eq:pl_comp}) involves the normalising constant for $\mathbf{z}_{B_i}$, which is a sum over all of the possible configurations $\mathcal{Z}_{B_i}$. This is a limiting factor on the size of blocks that can be used. The brute-force method that was used to compute Figure \ref{f:pl} is too computationally intensive for this purpose. \citet{Pettitt2003} showed that the normalising constant can be calculated exactly for a cylindrical lattice by computing eigenvalues of a $k^r \times k^r$ matrix, where $r$ is the smaller of the number of rows or columns. The value of (\ref{eq:norm}) for a free boundary lattice can then be approximated using path sampling. \citet{Friel2004} extended this method to larger lattices using a composite likelihood approach.

The reduced dependence approximation (RDA) is another form of composite likelihood. \citet{Reeves2004} introduced a recursive algorithm to calculate the normalising constant using a lag-$r$ representation. \citet{Friel2009} divided the image lattice into sub-lattices of size $r_1 < r$, then approximated the normalising constant of the full lattice using RDA:
\begin{equation}
\mathcal{C}(\beta) \approx \frac{\mathcal{C}_{r_1 \times n}(\beta)^{r - r_1 + 1}}{\mathcal{C}_{r_1 - 1 \times n}(\beta)^{r - r_1}}
\label{eq:rda}
\end{equation}
\citet{McGrory2009} compared RDA to pseudolikelihood and the exact method of \citet{Moeller2006}, reporting similar computational cost to pseudolikelihood but with improved accuracy in estimating $\beta$. \citet{Ogden2017} \dots

Source code for RDA is available in the online supplementary material for \citet{McGrory2012}.

# Examples
\label{s:results}

# Conclusion

\section*{Acknowledgements}

MTM was supported by the UK EPSRC as part of the *i*-Like programme grant (ref: EP/K014463/1). KLM was supported by an ARC Laureate Fellowship. KLM and ANP received funding from the ARC Centre of Excellence in Mathematical and Statistical Frontiers (ACEMS). Landsat imagery courtesy of NASA Goddard Space Flight Center and U.S. Geological Survey. Computational resources and services used in this work were provided by the HPC and Research Support Group, Queensland University of Technology, Brisbane, Australia.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{res2}
\alias{res2}
\title{Simulation from the Potts model using single-site Gibbs updates.}
\format{
A \code{list} containing 7 variables.
}
\usage{
res2
}
\description{
100 iterations of Gibbs sampling for a \eqn{500 \times 500} lattice
with \eqn{\beta=0.44} and \eqn{k=2}.
}
\seealso{
\code{\link{mcmcPotts}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBlocks.R
\name{getBlocks}
\alias{getBlocks}
\title{Get Blocks of a Graph}
\usage{
getBlocks(mask, nblock)
}
\arguments{
\item{mask}{a vector, matrix, or 3D array specifying vertices of a graph. Vertices of value 1 are within the graph and 0 are not.}

\item{nblock}{a scalar specifying the number of blocks. For a 2D graph \code{nblock} could be either 2 or 4, and for a 3D graph \code{nblock} could be either 2 or 8.}
}
\value{
A list with the number of components equal to \code{nblock}. Each component consists of vertices within the same block.
}
\description{
Obtain blocks of vertices of a 1D, 2D, or 3D graph, in order to use
  the conditional independence to speed up the simulation (chequerboard idea).
}
\details{
The vertices within each block are mutually independent given the vertices in other blocks. Some blocks could be empty.
}
\examples{
  #Example 1: split a line into 2 blocks
  getBlocks(mask=c(1,1,1,1,0,0,1,1,0), nblock=2)
  
  #Example 2: split a 4*4 2D graph into 4 blocks in order
  #           to use the chequerboard idea for a neighbourhood structure
  #           corresponding to the second-order Markov random field.
  getBlocks(mask=matrix(1, nrow=4, ncol=4), nblock=4)
  
  #Example 3: split a 3*3*3 3D graph into 8 blocks
  #           in order to use the chequerboard idea for a neighbourhood
  #           structure based on the 18 neighbors definition, where the
  #           neighbors of a vertex comprise its available
  #           adjacencies sharing the same edges or faces.
  mask <- array(1, dim=rep(3,3))
  getBlocks(mask, nblock=8)
}
\references{
Wilkinson, D. J. (2005)
    "Parallel Bayesian Computation"
    \cite{Handbook of Parallel Computing and Statistics}, pp. 481-512
    \emph{Marcel Dekker/CRC Press}
}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{swNoData}
\alias{swNoData}
\title{Simulate pixel labels using the Swendsen-Wang algorithm.}
\usage{
swNoData(beta, k, neighbors, blocks, niter = 1000, random = TRUE)
}
\arguments{
\item{beta}{The inverse temperature parameter of the Potts model.}

\item{k}{The number of unique labels.}

\item{neighbors}{A matrix of all neighbors in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{niter}{The number of iterations of the algorithm to perform.}

\item{random}{Whether to initialize the labels using random or deterministic starting values.}
}
\value{
A list containing the following elements:
\describe{
  \item{\code{alloc}}{An n by k matrix containing the number of times that pixel i was allocated to label j.}
  \item{\code{z}}{An \code{(n+1)} by k matrix containing the final sample from the Potts model after niter iterations of Swendsen-Wang.}
  \item{\code{sum}}{An \code{niter} by 1 matrix containing the sum of like neighbors, i.e. the sufficient statistic of the Potts model, at each iteration.}
  }
}
\description{
The algorithm of Swendsen & Wang (1987) forms clusters of neighbouring pixels,
then updates all of the labels within a cluster to the same value. When
simulating from the prior, such as a Potts model without an external field,
this algorithm is very efficient.
}
\examples{
# Swendsen-Wang for a 2x2 lattice
neigh <- matrix(c(5,2,5,3,  1,5,5,4,  5,4,1,5,  3,5,2,5), nrow=4, ncol=4, byrow=TRUE)
blocks <- list(c(1,4), c(2,3))
res.sw <- swNoData(0.7, 3, neigh, blocks, niter=200)
res.sw$z
res.sw$sum[200]
}
\references{
Swendsen, R. H. & Wang, J.-S. (1987) "Nonuniversal critical dynamics in Monte Carlo simulations" \emph{Physical Review Letters} \bold{58}(2), 86--88, DOI: \doi{10.1103/PhysRevLett.58.86}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smcPotts.R
\name{initSedki}
\alias{initSedki}
\title{Initialize the ABC algorithm using the method of Sedki et al. (2013)}
\usage{
initSedki(y, neighbors, blocks, param = list(npart = 10000), priors = NULL)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{neighbors}{A matrix of all neighbours in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{param}{A list of options for the ABC-SMC algorithm.}

\item{priors}{A list of priors for the parameters of the model.}
}
\value{
A matrix containing SMC samples for the parameters of the Potts model.
}
\description{
Initialize the ABC algorithm using the method of Sedki et al. (2013)
}
\references{
Sedki, M.; Pudlo, P.; Marin, J.-M.; Robert, C. P. & Cornuet, J.-M. (2013) "Efficient learning in ABC algorithms" \href{https://arxiv.org/abs/1210.1388}{arXiv:1210.1388}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smcPotts.R
\name{smcPotts}
\alias{smcPotts}
\title{Fit the hidden Potts model using approximate Bayesian computation with sequential Monte Carlo (ABC-SMC).}
\usage{
smcPotts(
  y,
  neighbors,
  blocks,
  param = list(npart = 10000, nstat = 50),
  priors = NULL
)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{neighbors}{A matrix of all neighbors in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{param}{A list of options for the ABC-SMC algorithm.}

\item{priors}{A list of priors for the parameters of the model.}
}
\value{
A matrix containing SMC samples for the parameters of the Potts model.
}
\description{
Fit the hidden Potts model using approximate Bayesian computation with sequential Monte Carlo (ABC-SMC).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smcPotts.R
\name{exactPotts}
\alias{exactPotts}
\title{Calculate the distribution of the Potts model using a brute force algorithm.}
\usage{
exactPotts(neighbors, blocks, k, beta)
}
\arguments{
\item{neighbors}{A matrix of all neighbours in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{k}{The number of unique labels.}

\item{beta}{The inverse temperature parameter of the Potts model.}
}
\value{
A list containing the following elements:
\describe{
  \item{\code{expectation}}{The exact mean of the sufficient statistic.}
  \item{\code{variance}}{The exact variance of the sufficient statistic.}
  \item{\code{exp_PL}}{Pseudo-likelihood (PL) approximation of the expectation of S(z).}
  \item{\code{var_PL}}{PL approx. of the variance of the sufficient statistic.}
  }
}
\description{
\bold{Warning}: this algorithm is O\eqn{(k^n)} and therefore will not scale for
\eqn{k^n > 2^{31} - 1}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayesImageS.R
\docType{package}
\name{bayesImageS}
\alias{bayesImageS-package}
\alias{bayesImageS}
\title{Package bayesImageS}
\description{
Bayesian methods for segmentation of 2D and 3D images, such as computed
tomography and satellite remote sensing. This package implements image
analysis using the hidden Potts model with external field prior. Latent labels
are sampled using chequerboard updating or Swendsen-Wang. Algorithms for the 
smoothing parameter include pseudolikelihood, path sampling, the exchange
algorithm, and approximate Bayesian computation (ABC-MCMC and ABC-SMC).
}
\references{
Moores, M. T.; Nicholls, G. K.; Pettitt, A. N. & Mengersen, K. (2020) "Scalable Bayesian inference for the inverse temperature of a hidden Potts model" \emph{Bayesian Analysis} \bold{15}(1), 1--17, DOI: \doi{10.1214/18-BA1130}

Moores, M. T.; Drovandi, C. C.; Mengersen, K. & Robert, C. P. (2015) "Pre-processing for approximate Bayesian computation in image analysis" \emph{Statistics & Computing} \bold{25}(1), 23--33, DOI: \doi{10.1007/s11222-014-9525-6}

Moores, M. T.; Hargrave, C. E.; Deegan, T.; Poulsen, M.; Harden, F. & Mengersen, K. (2015) "An external field prior for the hidden Potts model, with application to cone-beam computed tomography" \emph{Computational Statistics & Data Analysis} \bold{86}, 27--41, DOI: \doi{10.1016/j.csda.2014.12.001}

Feng, D. (2008) "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification" \emph{Ph. D. Dissertation, The University of Iowa}
}
\seealso{
\code{vignette(package="bayesImageS")}
}
\author{
M. T. Moores and K. Mengersen
with additional code contributed by D. Feng

Maintainer: Matt Moores <mmoores@gmail.com>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{sufficientStat}
\alias{sufficientStat}
\title{Calculate the sufficient statistic of the Potts model for the given labels.}
\usage{
sufficientStat(labels, neighbors, blocks, k)
}
\arguments{
\item{labels}{A matrix of pixel labels.}

\item{neighbors}{A matrix of all neighbors in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{k}{The number of unique labels.}
}
\value{
The sum of like neighbors.
}
\description{
Calculate the sufficient statistic of the Potts model for the given labels.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getEdges.R
\name{getEdges}
\alias{getEdges}
\title{Get Edges of a Graph}
\usage{
getEdges(mask, neiStruc)
}
\arguments{
\item{mask}{a vector, matrix, or 3D array specifying vertices of a graph. Vertices of value 1 are within the graph and 0 are not.}

\item{neiStruc}{a scalar, vector of four components, or \eqn{3\times4}    matrix corresponding to 1D, 2D, or 3D graphs. It specifies the neighbourhood structure. See \code{getNeighbors} for details.}
}
\value{
A matrix of two columns with one edge per row.  The edges connecting
vertices and their corresponding first neighbours are listed first, and
then those corresponding to the second neighbours, and so on and so
forth.  The order of neighbours is the same as in \code{getNeighbors}.
}
\description{
Obtain edges of a 1D, 2D, or 3D graph based on the
neighbourhood structure.
}
\details{
There could be more than one way to define the same 3D neighbourhood
structure for a graph (see Example 4 for illustration).
}
\examples{
  #Example 1: get all edges of a 1D graph. 
  mask <- c(0,0,rep(1,4),0,1,1,0,0)
  getEdges(mask, neiStruc=2)
  
  #Example 2: get all edges of a 2D graph based on neighbourhood structure
  #           corresponding to the first-order Markov random field.
  mask <- matrix(1 ,nrow=2, ncol=3)
  getEdges(mask, neiStruc=c(2,2,0,0))
  
  #Example 3: get all edges of a 2D graph based on neighbourhood structure
  #           corresponding to the second-order Markov random field.
  mask <- matrix(1 ,nrow=3, ncol=3)
  getEdges(mask, neiStruc=c(2,2,2,2))
  
  #Example 4: get all edges of a 3D graph based on 6 neighbours structure
  #           where the neighbours of a vertex comprise its available
  #           N,S,E,W, upper and lower adjacencies. To achieve it, there
  #           are several ways, including the two below.
  mask <- array(1, dim=rep(3,3))
  n61 <- matrix(c(2,2,0,0,
                  0,2,0,0,
                  0,0,0,0), nrow=3, byrow=TRUE)
  n62 <- matrix(c(2,0,0,0,
                  0,2,0,0,
                  2,0,0,0), nrow=3, byrow=TRUE)
  e1 <- getEdges(mask, neiStruc=n61)
  e2 <- getEdges(mask, neiStruc=n62)
  e1 <- e1[order(e1[,1], e1[,2]),]
  e2 <- e2[order(e2[,1], e2[,2]),]
  all(e1==e2)
  
  #Example 5: get all edges of a 3D graph based on 18 neighbours structure
  #           where the neighbours of a vertex comprise its available
  #           adjacencies sharing the same edges or faces.
  #           To achieve it, there are several ways, including the one below.
  
  n18 <- matrix(c(2,2,2,2,
                  0,2,2,2,
                  0,0,2,2), nrow=3, byrow=TRUE)  
  mask <- array(1, dim=rep(3,3))
  getEdges(mask, neiStruc=n18)
}
\references{
Winkler, G. (2003)
 "Image Analysis, Random Fields and Markov Chain Monte Carlo Methods: A Mathematical Introduction" (2nd ed.)
 \emph{Springer-Verlag}
 
 Feng, D. (2008)
 "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification"
 \emph{Ph. D. Dissertation, The University of Iowa}
}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{gibbsGMM}
\alias{gibbsGMM}
\title{Fit a mixture of Gaussians to the observed data.}
\usage{
gibbsGMM(y, niter = 1000, nburn = 500, priors = NULL)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{niter}{The number of iterations of the algorithm to perform.}

\item{nburn}{The number of iterations to discard as burn-in.}

\item{priors}{A list of priors for the parameters of the model.}
}
\value{
A matrix containing MCMC samples for the parameters of the mixture model.
}
\description{
Fit a mixture of Gaussians to the observed data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{gibbsNorm}
\alias{gibbsNorm}
\title{Fit a univariate normal (Gaussian) distribution to the observed data.}
\usage{
gibbsNorm(y, niter = 1000, priors = NULL)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{niter}{The number of iterations of the algorithm to perform.}

\item{priors}{A list of priors for the parameters of the model.}
}
\value{
A list containing MCMC samples for the mean and standard deviation.
}
\description{
Fit a univariate normal (Gaussian) distribution to the observed data.
}
\examples{
y <- rnorm(100,mean=5,sd=2)
res.norm <- gibbsNorm(y, priors=list(mu=0, mu.sd=1e6, sigma=1e-3, sigma.nu=1e-3))
summary(res.norm$mu[501:1000])
summary(res.norm$sigma[501:1000])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{synth}
\alias{synth}
\title{Simulation from the Potts model using Swendsen-Wang.}
\format{
A \code{list} containing 5 variables:
\describe{
  \item{0.22}{simulations for \eqn{\beta = 0.22}}
  \item{0.44}{simulations for \eqn{\beta = 0.44}}
  \item{0.88}{simulations for \eqn{\beta = 0.88}}
  \item{1.32}{simulations for \eqn{\beta = 1.32}}
  \item{tm}{time taken by the simulations}
}
}
\usage{
synth
}
\description{
Simulations for a \eqn{500 \times 500} lattice for fixed values
of the inverse temperature parameter, \eqn{\beta}.
}
\seealso{
\code{\link{swNoData}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNeighbors.R
\name{getNeighbors}
\alias{getNeighbors}
\title{Get Neighbours of All Vertices of a Graph}
\usage{
getNeighbors(mask, neiStruc)
}
\arguments{
\item{mask}{a vector, matrix, or 3D array specifying vertices within a graph. Vertices of value 1 are within the graph and 0 are not.}

\item{neiStruc}{a scalar, vector of four components, or \eqn{3\times4} matrix
corresponding to 1D, 2D, or 3D graphs. It gives the definition of
neighbours of a graph.
All components of \code{neiStruc} should be positive (\eqn{\ge 0})
even numbers.
For 1D graphs, \code{neiStruc} gives
the number of neighbours of each vertex.
For 2D graphs, \code{neiStruc}[1] specifies
the number of neighbours on vertical direction, \code{neiStruc}[2]
horizontal direction, \code{neiStruc}[3] north-west (NW) to south-east (SE)
diagonal direction, and \code{neiStruc}[4] south-west (SW) to
north-east (NE) diagonal direction. For 3D
graphs, the first row of \code{neiStruc} specifies the number of neighbours on
vertical direction, horizontal direction and two diagonal directions from
the 1-2 perspective, the second row the 1-3 perspective, and the
third row the 2-3 perspective. The index to
perspectives is represented with the leftmost subscript of the
array being the smallest.}
}
\value{
A matrix with each row giving the neighbours of a vertex.
The number of the rows is equal to the number of 
vertices within the graph and the number or columns
is the number of neighbours of each vertex.

For a 1D graph, if each vertex has two neighbours,
The first column are the neighbours on the left-hand side of
corresponding vertices and the second column the right-hand side.
For the vertices on boundaries, missing neighbours are represented by
the number of vertices within a graph plus 1.
When \code{neiStruc} is bigger than 2, The first two columns
are the same as when \code{neiStruc} is equal to 2; the third column
are the neighbours on the left-hand side of the vertices on the first column;
the forth column are the neighbours on the right-hand side of the vertices
on the second column, and so on and so forth. And again for the
vertices on boundaries, their missing neighbours are represented by
the number of vertices within a graph plus 1.

For a 2D graph, the index to vertices is column-wised. For each
vertex, the order of neighbours are as follows. First are those
on the vertical direction, second the horizontal
direction, third the NW to SE diagonal
direction, and forth the SW to NE diagonal
direction. For each direction, the neighbours of every vertex
are arranged in the same way as in a 1D graph.

For a 3D graph, the index to vertices is that
the leftmost subscript of the array moves the fastest.  For each
vertex, the neighbours from the 1-2 perspective
appear first and then the 1-3 perspective and finally
the 2-3 perspective. For each perspective, the neighbours are arranged
in the same way as in a 2D graph.
}
\description{
Obtain neighbours of vertices of a 1D, 2D, or 3D graph.
}
\details{
There could be more than one way to define the same 3D neighbourhood
structure for a graph (see Example 3 for illustration).
}
\examples{
  #Example 1: get all neighbours of a 1D graph.
  mask <- c(0,0,rep(1,4),0,1,1,0,0,1,1,1)
  getNeighbors(mask, neiStruc=2)
  
  #Example 2: get all neighbours of a 2D graph based on neighbourhood structure
  #           corresponding to the second-order Markov random field.
  mask <- matrix(1, nrow=2, ncol=3)
  getNeighbors(mask, neiStruc=c(2,2,2,2))
  
  #Example 3: get all neighbours of a 3D graph based on 6 neighbours structure
  #           where the neighbours of a vertex comprise its available
  #           N,S,E,W, upper and lower adjacencies. To achieve it, there
  #           are several ways, including the two below.
  mask <- array(1, dim=rep(3,3))
  n61 <- matrix(c(2,2,0,0,
                  0,2,0,0,
                  0,0,0,0), nrow=3, byrow=TRUE)
  n62 <- matrix(c(2,0,0,0,
                  0,2,0,0,
                  2,0,0,0), nrow=3, byrow=TRUE)
  n1 <- getNeighbors(mask, neiStruc=n61)
  n2 <- getNeighbors(mask, neiStruc=n62)
  n1 <- apply(n1, 1, sort)
  n2 <- apply(n2, 1, sort)
  all(n1==n2)
  
  #Example 4: get all neighbours of a 3D graph based on 18 neighbours structure
  #           where the neighbours of a vertex comprise its available
  #           adjacencies sharing the same edges or faces.
  #           To achieve it, there are several ways, including the one below.
  
  n18 <- matrix(c(2,2,2,2,
                  0,2,2,2,
                  0,0,2,2), nrow=3, byrow=TRUE)  
  mask <- array(1, dim=rep(3,3))
  getNeighbors(mask, neiStruc=n18)
}
\references{
Winkler, G. (2003)
 "Image Analysis, Random Fields and Markov Chain Monte Carlo Methods: A Mathematical Introduction" (2nd ed.)
 \emph{Springer-Verlag}
 
 Feng, D. (2008)
 "Bayesian Hidden Markov Normal Mixture Models with Application to MRI Tissue Classification"
 \emph{Ph. D. Dissertation, The University of Iowa}
}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{gibbsPotts}
\alias{gibbsPotts}
\title{Fit a hidden Potts model to the observed data, using a fixed value of beta.}
\usage{
gibbsPotts(y, labels, beta, mu, sd, neighbors, blocks, priors, niter = 1)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{labels}{A matrix of pixel labels.}

\item{beta}{The inverse temperature parameter of the Potts model.}

\item{mu}{A vector of means for the mixture components.}

\item{sd}{A vector of standard deviations for the mixture components.}

\item{neighbors}{A matrix of all neighbors in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{priors}{A list of priors for the parameters of the model.}

\item{niter}{The number of iterations of the algorithm to perform.}
}
\value{
A matrix containing MCMC samples for the parameters of the Potts model.
}
\description{
Fit a hidden Potts model to the observed data, using a fixed value of beta.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smcPotts.R
\name{testResample}
\alias{testResample}
\title{Test the residual resampling algorithm.}
\usage{
testResample(values, weights, pseudo)
}
\arguments{
\item{values}{A vector of SMC particles.}

\item{weights}{A vector of importance weights for each particle.}

\item{pseudo}{A matrix of pseudo-data for each particle.}
}
\value{
A list containing the following elements:
\describe{
  \item{\code{beta}}{A vector of resampled particles.}
  \item{\code{wt}}{The new importance weights, after resampling.}
  \item{\code{pseudo}}{A matrix of pseudo-data for each particle.}
  \item{\code{idx}}{The indices of the parents of the resampled particles.}
  }
}
\description{
Test the residual resampling algorithm.
}
\references{
Liu, J. S. & Chen, R. (1998) "Sequential Monte Carlo Methods for Dynamic Systems"
 \emph{J. Am. Stat. Assoc.} \bold{93}(443): 1032--1044, DOI: \doi{10.1080/01621459.1998.10473765}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmcPotts.R
\name{mcmcPotts}
\alias{mcmcPotts}
\title{Fit the hidden Potts model using a Markov chain Monte Carlo algorithm.}
\usage{
mcmcPotts(
  y,
  neighbors,
  blocks,
  priors,
  mh,
  niter = 55000,
  nburn = 5000,
  truth = NULL
)
}
\arguments{
\item{y}{A vector of observed pixel data.}

\item{neighbors}{A matrix of all neighbors in the lattice, one row per pixel.}

\item{blocks}{A list of pixel indices, dividing the lattice into independent blocks.}

\item{priors}{A list of priors for the parameters of the model.}

\item{mh}{A list of options for the Metropolis-Hastings algorithm.}

\item{niter}{The number of iterations of the algorithm to perform.}

\item{nburn}{The number of iterations to discard as burn-in.}

\item{truth}{A matrix containing the ground truth for the pixel labels.}
}
\value{
A matrix containing MCMC samples for the parameters of the Potts model.
}
\description{
Fit the hidden Potts model using a Markov chain Monte Carlo algorithm.
}
